<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Our main class
 *
 */
final class WC_IfthenPay_Webdados {
	
	/* Version */
	public $version = '3.5';

	/* IDs */
	public $multibanco_id = 'multibanco_ifthen_for_woocommerce';
	public $mbway_id = 'mbway_ifthen_for_woocommerce';

	/* Debug */
	public $log = null;

	/* Internal variables */
	public $wpml_active = false;
	public $wc_deposits_active = false;
	public $mb_ifthen_locale = null;
	public $out_link_utm = '';

	/* Internal variables - For Multibanco */
	public $multibanco_settings = null;
	public $multibanco_notify_url = '';
	public $multibanco_callback_email = 'callback@ifthenpay.com';
	public $multibanco_ents_no_check_digit = array( //Special entities with no check digit and (eventually) expiration date
		21721,
	);
	public $multibanco_ents_no_repeat = array( //Special entities with no repetition allowed in "x" days, no matter the order status - Only WooCommerce 3.0 and above
		'11687' => 180,
	);
	public $multibanco_action_deposits_set = false;
	public $multibanco_deposits_already_forced = false;


	/* Internal variables - For MB WAY */
	public $mbway_settings = null;
	public $mbway_notify_url = '';
	public $mbway_callback_email = 'callback@ifthenpay.com';
	public $mbway_minutes = 5;
	public $mbway_multiplier_new_payment = 1.2;

	/* Internal variables - This is here because on the main class duplication still happens - Fixed by checking class instances */
	//public $instructions_sent_to_client = false;
	//public $instructions_sent_to_admin = false;

	/* Single instance */
	protected static $_instance = null;

	/* Constructor */
	public function __construct() {
		$this->wpml_active = function_exists( 'icl_object_id' ) && function_exists( 'icl_register_string' );
		$this->wc_deposits_active = function_exists( 'wc_deposits_woocommerce_is_active' );
		$this->out_link_utm = '?utm_source='.rawurlencode( esc_url( home_url( '/' ) ) ).'&amp;utm_medium=link&amp;utm_campaign=mb_ifthen_plugin';
		$this->multibanco_settings = get_option( 'woocommerce_multibanco_ifthen_for_woocommerce_settings', '' );
		$this->multibanco_notify_url = (
									get_option( 'permalink_structure' )==''
									?
									home_url( '/?wc-api=WC_Multibanco_IfThen_Webdados&chave=[CHAVE_ANTI_PHISHING]&entidade=[ENTIDADE]&referencia=[REFERENCIA]&valor=[VALOR]&datahorapag=[DATA_HORA_PAGAMENTO]&terminal=[TERMINAL]' )
									:
									home_url( '/wc-api/WC_Multibanco_IfThen_Webdados/?chave=[CHAVE_ANTI_PHISHING]&entidade=[ENTIDADE]&referencia=[REFERENCIA]&valor=[VALOR]&datahorapag=[DATA_HORA_PAGAMENTO]&terminal=[TERMINAL]' )
								);
		$this->mbway_settings = get_option( 'woocommerce_mbway_ifthen_for_woocommerce_settings', '' );
		$this->mbway_notify_url = (
									get_option( 'permalink_structure' )==''
									?
									home_url( '/?wc-api=WC_MBWAY_IfThen_Webdados&chave=[CHAVE_ANTI_PHISHING]&referencia=[REFERENCIA]&idpedido=[ID_TRANSACAO]&valor=[VALOR]&datahorapag=[DATA_HORA_PAGAMENTO]&estado=[ESTADO]' )
									:
									home_url( '/wc-api/WC_MBWAY_IfThen_Webdados/?chave=[CHAVE_ANTI_PHISHING]&referencia=[REFERENCIA]&idpedido=[ID_TRANSACAO]&valor=[VALOR]&datahorapag=[DATA_HORA_PAGAMENTO]&estado=[ESTADO]' )
								);
		$this->init_hooks();
	}

	/* Ensures only one instance of our plugin is loaded or can be loaded */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	/* Hooks */
	private function init_hooks() {
		add_filter( 'woocommerce_payment_gateways', array( $this, 'woocommerce_add_payment_gateways' ) );
		add_action( 'add_meta_boxes', array( $this, 'multibanco_order_metabox' ) );
		add_filter( 'woocommerce_shop_order_search_fields', array( $this, 'multibanco_shop_order_search' ) );
		add_action( 'wp_ajax_multibanco_ifthen_dismiss_callback_notice', array( $this, 'multibanco_dismiss_callback_notice' ) );
		add_action( 'wp_ajax_mbway_ifthen_dismiss_callback_notice', array( $this, 'mbway_dismiss_callback_notice' ) );
		add_action( 'woocommerce_checkout_update_order_meta', array( $this, 'multibanco_woocommerce_checkout_update_order_meta' ) ); 	//Frontend
		add_action( 'plugins_loaded', array( $this, 'multibanco_woocommerce_process_shop_order_meta_backend' ) );
		add_action( 'woocommerce_saved_order_items', array( $this, 'multibanco_woocommerce_saved_order_items' ) );
		add_filter( 'woocommerce_order_data_store_cpt_get_orders_query', array( $this, 'multibanco_woocommerce_order_data_store_cpt_get_orders_query' ), 10, 2 );
		add_action( 'woocommerce_cancel_unpaid_orders', array( $this, 'multibanco_woocommerce_cancel_unpaid_orders' ), 99 );
		add_filter( 'apg_sms_message', array( $this, 'multibanco_apg_sms_message' ), 10, 2 );
		add_filter( 'wcs_renewal_order_meta', array( $this, 'multibanco_wcs_filter_meta' ) );
		add_filter( 'wcs_resubscribe_order_meta', array( $this, 'multibanco_wcs_filter_meta' ) );
		add_filter( 'wcs_renewal_order_created', array( $this, 'multibanco_wcs_renewal_order_created' ), 11, 2 );
		add_action( 'woocommerce_send_queued_transactional_email', array( $this, 'woocommerce_send_queued_transactional_email' ), 1, 2 );
		add_action( 'plugins_loaded', array( $this, 'wpml_ajax_fix_locale' ) );
		add_action( 'woocommerce_new_customer_note', array( $this, 'woocommerce_new_customer_note_fix_wpml' ), 1 );
		// Order status listener/Ajax hook
		add_action( 'wp_ajax_wc_mbway_ifthen_order_status', array( $this, 'mbway_ajax_order_status' ) );
		// Request MB WAY payment again
		add_action( 'wp_ajax_mbway_ifthen_request_payment_again', array( $this, 'wp_ajax_mbway_ifthen_request_payment_again' ) );
	}

	/* Add settings link to plugin actions */
	public function add_settings_link( $links ) {
		$action_links = array(
			'mb_settings'		=> '<a href="admin.php?page=wc-settings&amp;tab=checkout&amp;section='.$this->multibanco_id.'">' . __( 'Multibanco settings', 'multibanco-ifthen-software-gateway-for-woocommerce' ) . '</a>',
			'mbway_settings'	=> '<a href="admin.php?page=wc-settings&amp;tab=checkout&amp;section='.$this->mbway_id.'">' . __( 'MB WAY settings', 'multibanco-ifthen-software-gateway-for-woocommerce' ) . '</a>',
		);
		return array_merge( $action_links, $links );
	}

	/* Add to WooCommerce */
	public function woocommerce_add_payment_gateways( $methods ) {
		$methods[] = 'WC_Multibanco_IfThen_Webdados';
		$methods[] = 'WC_MBWAY_IfThen_Webdados';
		return $methods;
	}

	/* Debug / Log */
	public function debug_log( $gateway_id, $message, $level = 'debug', $debug_email = '', $email_message = '' ) {
		if ( !$this->log ) $this->log = version_compare( WC_VERSION, '3.0', '>=' ) ? wc_get_logger() : new WC_Logger(); //Init log 
		if ( version_compare( WC_VERSION, '3.0', '>=' ) ) {
			$this->log->$level( $message, array( 'source' => $gateway_id ) );
		} else {
			$this->log->add( $gateway_id, $message );
		}
		if ( $debug_email ) {
			wp_mail(
				trim( $debug_email ),
				$gateway_id.' - '.$message,
				$email_message
			);
		}
	}

	/* Format MB reference */
	public function format_multibanco_ref( $ref ) {
		return apply_filters( 'multibanco_ifthen_format_ref', trim( chunk_split( trim( $ref ), 3, ' ' ) ) );
	}


	/* Disable payment gateway if not € */
	public function disable_if_currency_not_euro( $available_gateways, $gateway_id ) {
		if ( isset( $available_gateways[$gateway_id] ) ) {
			if ( trim( get_woocommerce_currency() ) != 'EUR' ) unset( $available_gateways[$gateway_id] );
		}
		return $available_gateways;
	}

	/* Customer billing country */
	public function get_customer_billing_country() {
		if ( version_compare( WC_VERSION, '3.0', '>=' ) ) {
			return trim( WC()->customer->get_billing_country() );
		} else {
			return trim( WC()->customer->get_country() );
		}
	}

	/* Disable unless Portugal */
	public function disable_unless_portugal( $available_gateways, $gateway_id ) {
		if ( isset( $available_gateways[$gateway_id] ) ) {
			if ( $available_gateways[$gateway_id]->only_portugal && WC()->customer && $this->get_customer_billing_country() != 'PT' ) unset( $available_gateways[$gateway_id] );
		}
		return $available_gateways;
	}

	/* Just above/bellow certain amounts */
	public function disable_only_above_or_bellow( $available_gateways, $gateway_id ) {
		if ( isset( $available_gateways[$gateway_id] ) ) {
			if ( @floatval( $available_gateways[$gateway_id]->only_above ) > 0 ) {
				if( WC()->cart->total < floatval( $available_gateways[$gateway_id]->only_above ) ) {
					unset( $available_gateways[$gateway_id] );
				}
			} 
			if ( @floatval( $available_gateways[$gateway_id]->only_bellow) > 0 ) {
				if( WC()->cart->total>floatval( $available_gateways[$gateway_id]->only_bellow ) ) {
					unset( $available_gateways[$gateway_id] );
				}
			} 
		}
		return $available_gateways;
	}

	/* Get multibanco order details */
	public function get_multibanco_order_details( $order_id ) {
		$order = new WC_Order_MB_Ifthen( $order_id );
		$ent = $order->mb_get_meta( '_'.$this->multibanco_id.'_ent' );
		$ref = $order->mb_get_meta( '_'.$this->multibanco_id.'_ref' );
		$val = $order->mb_get_meta( '_'.$this->multibanco_id.'_val' );
		if ( !empty( $ent ) &&  !empty( $ref ) &&  !empty( $val ) ) {
			return array(
						'ent'	=> $ent,
						'ref'	=> $ref,
						'val'	=> $val,
					);
		}
		return false;
	}

	/* Get mb way order details */
	public function get_mbway_order_details( $order_id ) {
		$order = new WC_Order_MB_Ifthen( $order_id );
		$mbwaykey = $order->mb_get_meta( '_'.$this->mbway_id.'_mbwaykey' );
		$id_pedido = $order->mb_get_meta( '_'.$this->mbway_id.'_id_pedido' );
		//$phone = $order->mb_get_meta( '_'.$this->mbway_id.'_phone' ); //GDPR 2018-06-18
		$val = $order->mb_get_meta( '_'.$this->mbway_id.'_val' );
		if ( !empty( $mbwaykey ) && !empty( $id_pedido )  && !empty( $val ) ) {
			return array(
						'mbwaykey'	=> $mbwaykey,
						'id_pedido'	=> $id_pedido,
						//'phone'	=> $phone, //GDPR 2018-06-18
						'val'	=> $val,
					);
		}
		return false;
	}

	/* Order metabox to show Multibanco payment details - This will need to change when the order is no longer a WP post */
	public function multibanco_order_metabox() {
		add_meta_box( $this->multibanco_id, __( 'Multibanco or MB WAY payment details', 'multibanco-ifthen-software-gateway-for-woocommerce' ), array( $this, 'multibanco_order_metabox_html' ), 'shop_order', 'side', 'core' );
	}
	public function multibanco_order_metabox_html( $post ) {
		$order = new WC_Order_MB_Ifthen( $post->ID );
		//MULTIBANCO
		if ( $order->mb_get_payment_method() == $this->multibanco_id ) {
			if (
				$order_mb_details = $this->get_multibanco_order_details( $order->mb_get_id() )
			) {
				echo '<p><strong>'.__( 'Multibanco', 'multibanco-ifthen-software-gateway-for-woocommerce' ).'</strong></p>';
				echo '<p>'.__( 'Entity', 'multibanco-ifthen-software-gateway-for-woocommerce' ).': '.trim( $order_mb_details['ent'] ).'<br/>';
				echo __( 'Reference', 'multibanco-ifthen-software-gateway-for-woocommerce' ).': '.$this->format_multibanco_ref( $order_mb_details['ref'] ).'<br/>';
				echo __( 'Value', 'multibanco-ifthen-software-gateway-for-woocommerce' ).': '.wc_price( $order_mb_details['val'] ).'</p>';
				if ( $order->mb_has_status( 'on-hold' ) || $order->mb_has_status( 'pending' ) ) {
					$show_debug = true;
					if ( $this->wc_deposits_active && ( $order->mb_get_status() == 'partially-paid' || ( $order->mb_get_status() == 'on-hold' && $order->mb_get_meta( '_wc_deposits_deposit_paid' ) == 'yes' ) ) ) {
						echo '<p>'.__( 'Partially paid.', 'multibanco-ifthen-software-gateway-for-woocommerce' ).'</p>';
						if ( $order->mb_get_meta( '_wc_deposits_second_payment_paid' ) != 'yes' &&  floatval( $order->mb_get_meta( '_wc_deposits_second_payment' ) ) == floatval( $order_mb_details['val'] )  ) {
							echo '<p>'.__( 'Awaiting second Multibanco payment.', 'multibanco-ifthen-software-gateway-for-woocommerce' ).'</p>';
						} else {
							$show_debug = false;
						}
					} else {
						echo '<p>'.__( 'Awaiting Multibanco payment.', 'multibanco-ifthen-software-gateway-for-woocommerce' ).'</p>';
					}
					if ( $show_debug && WP_DEBUG ) {
						$callback_url = $this->multibanco_notify_url;
						$callback_url = str_replace( '[CHAVE_ANTI_PHISHING]', $this->multibanco_settings['secret_key'], $callback_url );
						$callback_url = str_replace( '[ENTIDADE]', trim( $order_mb_details['ent'] ), $callback_url );
						$callback_url = str_replace( '[REFERENCIA]', trim( $order_mb_details['ref'] ), $callback_url );
						$callback_url = str_replace( '[VALOR]', $order_mb_details['val'], $callback_url );
						$callback_url = str_replace( '[DATA_HORA_PAGAMENTO]', '', $callback_url );
						$callback_url = str_replace( '[TERMINAL]', 'Testing', $callback_url );
						?>
						<hr/>
						<p>
							<?php _e( 'Callback URL', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>:<br/>
							<textarea readonly type="text" class="input-text" cols="20" rows="5" style="width: 100%; height: 50%; font-size: 10px;"><?php echo $callback_url; ?></textarea>
						</p>
						<script type="text/javascript">
						jQuery( document ).ready( function() {
							jQuery( '#multibanco_ifthen_for_woocommerce_simulate_callback' ).click( function() {
								if ( confirm( '<?php _e( 'This is a testing tool and will set the order as paid. Are you sure you want to proceed?', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>' ) ) {
									jQuery.get( '<?php echo $callback_url; ?>', '', function( response ) {
										alert( '<?php _e( 'This page will now reload. If the order is not set as paid and processing (or completed, if it only contains virtual and downloadable products) please check the debug logs.', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>' );
										window.location.reload();
									}).fail( function() {
										alert( '<?php _e( 'Error: Could not set the order as paid', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>' );
									});
								}
							});
						});
						</script>
						<p align="center">
							<input type="button" class="button" id="multibanco_ifthen_for_woocommerce_simulate_callback" value="<?php echo esc_attr(__( 'Simulate callback payment', 'multibanco-ifthen-software-gateway-for-woocommerce' ) ); ?>"/>
						</p>
						<?php
					}
				} else {
					//PAID?
				}
			} else {
				echo '<p>'.__( 'No details available', 'multibanco-ifthen-software-gateway-for-woocommerce' ).'.</p><p>'.__( 'This must be an error because the payment method of this order is Multibanco', 'multibanco-ifthen-software-gateway-for-woocommerce' ).'.</p>';
			}
		} else {
			if ( $order->mb_get_payment_method() == $this->mbway_id ) {
				if (
					$order_mbway_details = $this->get_mbway_order_details( $order->mb_get_id() )
				) {
					echo '<p><strong>'.__( 'MB WAY', 'multibanco-ifthen-software-gateway-for-woocommerce' ).'</strong></p>';
					if ( $order->mb_has_status( 'on-hold' ) || $order->mb_has_status( 'pending' ) ) {
						$show_debug = true;
						if ( $this->wc_deposits_active && $order->mb_get_status() == 'partially-paid' ) {
							echo '<p>'.__( 'Partially paid.', 'multibanco-ifthen-software-gateway-for-woocommerce' ).'</p>';
							if ( $order->mb_get_meta( '_wc_deposits_second_payment_paid' ) != 'yes' &&  floatval( $order->mb_get_meta( '_wc_deposits_second_payment' ) ) == floatval( $order_mbway_details['val'] )  ) {
								echo '<p>'.__( 'Awaiting second MB WAY payment.', 'multibanco-ifthen-software-gateway-for-woocommerce' ).'</p>';
							} else {
								$show_debug = false;
							}
						} else {
							echo '<p>'.__( 'Awaiting MB WAY payment.', 'multibanco-ifthen-software-gateway-for-woocommerce' ).'</p>';
							if ( date_i18n( 'Y-m-d H:i:s', strtotime( '-'.( $this->mbway_minutes * $this->mbway_multiplier_new_payment ).' MINUTES', current_time( 'timestamp' ) ) ) > $order->mb_get_meta( '_'.$this->mbway_id.'_time' ) ) {
								?>
								<p align="center">
									<input type="button" class="button" id="mbway_ifthen_request_payment_again" value="<?php echo esc_attr( __( 'Request MB WAY payment again', 'multibanco-ifthen-software-gateway-for-woocommerce' ) ); ?>"/>
								</p>
								<script type="text/javascript">
								jQuery( document ).ready( function() {
									jQuery( '#mbway_ifthen_request_payment_again' ).click( function() {
										if ( confirm( '<?php echo __( "Are you sure you want to request the payment again? Don\'t do it unless your client asks you to.", 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>' ) ) {
											jQuery( '#mbway_ifthen_request_payment_again' ).val( '<?php _e( 'Wait...', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>' );
											var phone = '<?php echo $order->mb_get_meta( '_'.$this->mbway_id.'_phone' ); ?>';
											if ( phone == '' ) phone = prompt( '<?php echo __( 'MB WAY phone number', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>', '' );
											if ( phone ) {
												var data = {
													'action'  : 'mbway_ifthen_request_payment_again',
													'order_id': <?php echo $order->mb_get_id(); ?>,
													'nonce'   : '<?php echo wp_create_nonce( 'mbway_ifthen_request_payment_again' ); ?>',
													'phone'   : phone
												};
												jQuery.post( ajaxurl, data, function( response ) {
													if ( response.status == 1 ) {
														jQuery( '#mbway_ifthen_request_payment_again' ).val( '<?php _e( 'Done!', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>' );
														alert( response.message );
														window.location.reload();
													} else {
														jQuery( '#mbway_ifthen_request_payment_again' ).val( '<?php _e( 'Request MB WAY payment again', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>' );
														alert( response.error );
													}
												}, 'json' ).fail( function() {
													jQuery( '#mbway_ifthen_request_payment_again' ).val( '<?php _e( 'Request MB WAY payment again', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>' );
													alert( '<?php _e( 'Unknown error.', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>' );
												} );
											}
										}
									});
								});
								</script>
								<?php
							}
						}
						if ( $show_debug && WP_DEBUG ) {
							$callback_url = $this->mbway_notify_url;
							$callback_url = str_replace( '[CHAVE_ANTI_PHISHING]', $this->mbway_settings['secret_key'], $callback_url );
							$callback_url = str_replace( '[REFERENCIA]', $order->mb_get_id(), $callback_url );
							$callback_url = str_replace( '[ID_TRANSACAO]', trim( $order_mbway_details['id_pedido'] ), $callback_url );
							$callback_url = str_replace( '[VALOR]', $order_mbway_details['val'], $callback_url );
							$callback_url = str_replace( '[DATA_HORA_PAGAMENTO]', '', $callback_url );
							$callback_url = str_replace( '[ESTADO]', 'PAGO', $callback_url );
							?>
							<hr/>
							<p>
								<?php _e( 'Callback URL', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>:<br/>
								<textarea readonly type="text" class="input-text" cols="20" rows="5" style="width: 100%; height: 50%; font-size: 10px;"><?php echo $callback_url; ?></textarea>
							</p>
							<script type="text/javascript">
							jQuery( document ).ready( function() {
								jQuery( '#multibanco_ifthen_for_woocommerce_simulate_callback' ).click( function() {
									if ( confirm( '<?php _e( 'This is a testing tool and will set the order as paid. Are you sure you want to proceed?', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>' ) ) {
										jQuery.get( '<?php echo $callback_url; ?>', '', function( response ) {
											alert( '<?php _e( 'This page will now reload. If the order is not set as paid and processing (or completed, if it only contains virtual and downloadable products) please check the debug logs.', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>' );
											window.location.reload();
										}).fail( function() {
											alert( '<?php _e( 'Error: Could not set the order as paid', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>' );
										});
									}
								});
							});
							</script>
							<p align="center">
								<input type="button" class="button" id="multibanco_ifthen_for_woocommerce_simulate_callback" value="<?php echo esc_attr(__( 'Simulate callback payment', 'multibanco-ifthen-software-gateway-for-woocommerce' ) ); ?>"/>
							</p>
							<?php
						}
					} else {
						//PAID?
					}
				} else {
					echo '<p>'.__( 'No details available', 'multibanco-ifthen-software-gateway-for-woocommerce' ).'.</p><p>'.__( 'This must be an error because the payment method of this order is MB WAY', 'multibanco-ifthen-software-gateway-for-woocommerce' ).'.</p>';
				}
			} else {
				echo '<p>'.__( 'No details available', 'multibanco-ifthen-software-gateway-for-woocommerce' ).'.</p><p>'.__( 'The payment method of this order is not Multibanco or MB WAY', 'multibanco-ifthen-software-gateway-for-woocommerce' ).'.</p>';
				echo '<style type="text/css">#'.$this->multibanco_id.' { display: none; }</style>';
				//If we have MB data, we should delete it
				if ( $order_mb_details = $this->get_multibanco_order_details( $order->mb_get_id() ) ) {
					foreach ( $order_mb_details as $key => $value ) {
						$order->mb_delete_meta_data( '_'.$this->multibanco_id.'_'.$key );
					}
				}
				if ( $order_mb_details = $this->get_mbway_order_details( $order->mb_get_id() ) ) {
					foreach ( $order_mb_details as $key => $value ) {
						$order->mb_delete_meta_data( '_'.$this->mbway_id.'_'.$key );
					}
				}
			}
		}
	}

	/* Allow searching orders by Multibanco reference */
	public function multibanco_shop_order_search( $search_fields ) {
		$search_fields[] = '_'.$this->multibanco_id.'_ref';
		return $search_fields;
	}

	/* Dismiss callback notice */
	public function multibanco_dismiss_callback_notice() {
		update_option( $this->multibanco_id.'_callback_notice_dismiss', 'yes' );
		die( '1' );
	}
	public function mbway_dismiss_callback_notice() {
		update_option( $this->mbway_id.'_callback_notice_dismiss', 'yes' );
		die( '1' );
	}

	/* Set new order Entity/Reference/Value on meta */
	public function multibanco_set_order_mb_details( $order_id, $order_mb_details ) {
		$order = new WC_Order_MB_Ifthen( $order_id );
		$order->mb_update_meta_data( '_'.$this->multibanco_id.'_ent', $order_mb_details['ent'] );
		$order->mb_update_meta_data( '_'.$this->multibanco_id.'_ref', $order_mb_details['ref'] );
		$order->mb_update_meta_data( '_'.$this->multibanco_id.'_val', $order_mb_details['val'] );
	}

	/* Set new order MB WAY Deatils on meta */
	public function multibanco_set_order_mbway_details( $order_id, $order_mbway_details ) {
		$order = new WC_Order_MB_Ifthen( $order_id );
		$order->mb_update_meta_data( '_'.$this->mbway_id.'_mbwaykey', $order_mbway_details['mbwaykey'] );
		$order->mb_update_meta_data( '_'.$this->mbway_id.'_id_pedido', $order_mbway_details['id_pedido'] );
		$order->mb_update_meta_data( '_'.$this->mbway_id.'_phone', $order_mbway_details['phone'] );
		$order->mb_update_meta_data( '_'.$this->mbway_id.'_val', $order_mbway_details['val'] );
		$order->mb_update_meta_data( '_'.$this->mbway_id.'_time', date_i18n( 'Y-m-d H:i:s' ) );
	}

	/* Get/Create Multibanco Reference */
	public function multibanco_get_ref( $order_id, $force_change = false ) {
		$order = new WC_Order_MB_Ifthen( $order_id );
		if ( $this->wc_deposits_active ) {
			if ( ! $this->multibanco_deposits_already_forced ) {
				if ( $order->mb_get_meta( '_wc_deposits_order_has_deposit' ) == 'yes' && is_checkout() && has_action( 'woocommerce_thankyou' ) ) {
					if ( $order->mb_get_meta( '_wc_deposits_deposit_paid' ) == 'yes' ) {
						if ( $order->mb_get_meta( '_wc_deposits_second_payment_paid' ) == 'no' ) {
							$force_change = true;
							$this->multibanco_deposits_already_forced = true;
						}
					}
				}
			}
		}
		if ( trim( get_woocommerce_currency() ) == 'EUR' ) {
			if (
				!$force_change
				&&
				$order_mb_details = $this->get_multibanco_order_details( $order_id )

			) {
				//Already created, return it!
				return array(
					'ent' => $order_mb_details['ent'],
					'ref' => $order_mb_details['ref'],
				);
			} else {
				//Value ok?
				if ( $order->mb_get_total() < 1 ){
					return __( 'It\'s not possible to use Multibanco to pay values under 1&euro;.', 'multibanco-ifthen-software-gateway-for-woocommerce' );
			 	} else {
			 		//Value ok? (again)
					if ( $order->mb_get_total() >= 1000000 ){
						return __( 'It\'s not possible to use Multibanco to pay values above 999999&euro;.', 'multibanco-ifthen-software-gateway-for-woocommerce' );
					} else {
						//Create a new reference
						//Filters to be able to override the Entity and Sub-entity - Can be usefull for marketplaces
						$base = apply_filters( 'multibanco_ifthen_base_ent_subent', array( 'ent' => $this->multibanco_settings['ent'], 'subent' => $this->multibanco_settings['subent'] ), $order );
						if (
							strlen( trim( $base['ent'] ) ) == 5
							&&
							strlen( trim( $base['subent'] ) ) <= 3
							&&
							intval( $base['ent'] ) > 0
							&&
							intval( $base['subent'] ) > 0
							&&
							trim( $this->multibanco_settings['secret_key'] ) != ''
						) {
							if (  version_compare( WC_VERSION, '3.0', '>=' ) && isset( $this->multibanco_ents_no_repeat[ $base['ent'] ] ) && intval( $this->multibanco_ents_no_repeat[ $base['ent'] ] ) > 0 ) {
								//No repeat in x days
								$ref = $this->multibanco_create_ref( $base['ent'], $base['subent'], rand(0,9999), $order->mb_get_total(), intval( $this->multibanco_ents_no_repeat[ $base['ent'] ] ) );
							} else {
								if ( in_array( intval( $base['ent'] ), $this->multibanco_ents_no_check_digit ) && ( $this->multibanco_settings['use_order_id'] =='yes' ) ) {
									//Special entities with no check digit and (eventually) expiration date - We can use the order ID
									$ref = $this->multibanco_create_ref_no_check_digit( $base['ent'], $base['subent'], $order_id, $order->mb_get_total() );
								} else {
									//$ref=$this->multibanco_create_ref( $base['ent'], $base['subent'], 0, $order->mb_get_total() ); //For incremental mode
									$ref = $this->multibanco_create_ref( $base['ent'], $base['subent'], rand(0,9999), $order->mb_get_total() ); //For random mode - Less loop possibility
								}
							}
							//Store on the order for later use (like the email)
							$this->multibanco_set_order_mb_details( $order_id, array(
								'ent'	=>	$base['ent'],
								'ref'	=>	$ref,
								'val'	=>	$order->mb_get_total(),
							) );
							$this->debug_log( $this->multibanco_id, 'Multibanco payment details ('.$base['ent'].' / '.$ref.' / '.$order->mb_get_total().') generated for Order '.$order_id );
							//Return it!
							do_action( 'multibanco_ifthen_created_reference', array(
								'ent' => $base['ent'],
								'ref' => $ref
							), $order_id, $force_change );
							//WooCommerce Deposits support - force ref creation again
							if ( ! $force_change && $this->wc_deposits_active && ! $this->multibanco_action_deposits_set ) {
								add_action( 'woocommerce_checkout_order_processed', array( $this, 'multibanco_get_ref_deposit' ), 20, 1 );
								$this->debug_log( $this->multibanco_id, 'Because of WooCommerce Deposits a new reference will be generated for Order '.$order_id );
								$this->multibanco_action_deposits_set = true;
							}
							return array(
								'ent' => $base['ent'],
								'ref' => $ref
							);
						} else {
							$error_details='';
							if ( trim( strlen( $base['ent'] ) ) != 5 || ( !intval( $base['ent'] ) > 0 ) ) {
								$error_details = __( 'Entity', 'multibanco-ifthen-software-gateway-for-woocommerce' );
							} else {
								if ( trim( strlen( $base['subent'] ) ) != 5 || ( !intval( $base['subent'] ) > 0 ) ) {
									$error_details = __( 'Subentity', 'multibanco-ifthen-software-gateway-for-woocommerce' );
								} else {
									if ( trim( $this->multibanco_settings['secret_key'] ) == '' ) {
										$error_details = __( 'Anti-phishing key', 'multibanco-ifthen-software-gateway-for-woocommerce' );
									}
								}
							}
							return __( 'Configuration error. This payment method is disabled because required information was not set.', 'multibanco-ifthen-software-gateway-for-woocommerce' ).' '.$error_details.'.';
						}
					}
				}
			}
		} else {
			return __( 'Configuration error. This store currency is not Euros (&euro;).', 'multibanco-ifthen-software-gateway-for-woocommerce' );
		}
	}
	public function multibanco_get_ref_deposit( $order_id ) {
		//WooCommerce Deposits support - force ref creation again
		if ( $this->wc_deposits_active ) $ref = WC_IfthenPay_Webdados()->multibanco_get_ref( $order_id, true );
	}
	public function multibanco_create_ref( $ent, $subent, $seed, $total, $no_repeat_days = 0, $just_create_no_check = false ) {
		$subent = str_pad( intval( $subent ), 3, "0", STR_PAD_LEFT );
		$seed = str_pad( intval( $seed ), 4, "0", STR_PAD_LEFT );
		$chk_str = sprintf( '%05u%03u%04u%08u', $ent, $subent, $seed, round( $total*100 ) );
		$chk_array = array( 3, 30, 9, 90, 27, 76, 81, 34, 49, 5, 50, 15, 53, 45, 62, 38, 89, 17, 73, 51 );
		$chk_val = 0;
		for ( $i = 0; $i < 20; $i++ ) {
			$chk_int = substr( $chk_str, 19-$i, 1 );
			$chk_val += ( $chk_int%10 ) * $chk_array[$i];
		}
		$chk_val %= 97;
		$chk_digits = sprintf( '%02u', 98-$chk_val );
		$ref = $subent.$seed.$chk_digits;
		//Does it exists already? Let's browse the database!
		if ( ! $just_create_no_check ) {
			$exists = false;
			if ( version_compare( WC_VERSION, '3.0', '<' ) ) {
				//Old way
				if ( version_compare( WC_VERSION, '2.2', '<' ) ) {
					//The old way before WooCommerce 2.2
					$args = array(
						'post_type'	=> 'shop_order',
						'post_status' => 'publish',
						'posts_per_page' => 1, //If there's one, it's enough
						'tax_query' => array(
							//We should also look for pending because of the filter added on 2017-12-15, but people should not be using WC < 2.2 by now...
							array(
								'taxonomy' => 'shop_order_status',
								'field' => 'slug',
								'terms' => array( 'on-hold', 'pending' ),
							)
						),
						'meta_query' => array(
							array(
								'key' => '_'.$this->multibanco_id.'_ent',
								'value' => $ent,
								'compare' => 'LIKE'
							),
							array(
								'key' => '_'.$this->multibanco_id.'_ref',
								'value' => $ref,
								'compare' => 'LIKE'
							)
						)
					);
				} else {
					//The old way From WooCommerce 2.2
					$args = array(
						'post_type' => 'shop_order',
						'post_status' => array( 'wc-on-hold', 'wc-pending' ),
						'posts_per_page' => 1, //If there's one, it's enough
						'meta_query' => array(
							array(
								'key' => '_'.$this->multibanco_id.'_ent',
								'value' => $ent,
								'compare' => 'LIKE'
							),
							array(
								'key' => '_'.$this->multibanco_id.'_ref',
								'value' => $ref,
								'compare' => 'LIKE'
							)
						)
					);
				}
				$the_query = new WP_Query( $args );
				if ( $the_query->have_posts() ) $exists = true;
				wp_reset_postdata();
			} else {
				//New way
				$orders = wc_get_orders( array(
					'type'	=> array( 'shop_order' ),
					'limit'	=> 1, //If there's one, it's enough
					'_'.$this->multibanco_id.'_ent' => $ent,
					'_'.$this->multibanco_id.'_ref' => $ref,
					'status' => array( 'wc-on-hold', 'wc-pending' ),
				) );
				if ( count($orders) > 0 ) {
					$exists = true;
				} else {
					//No open orders but also check for special entities that do not allow references to be repeated on x days
					if ( intval( $no_repeat_days ) > 0 ) {
						$orders = wc_get_orders( array(
							'type'	=> array( 'shop_order' ),
							'limit'	=> 1, //If there's one, it's enough
							'_'.$this->multibanco_id.'_ent' => $ent,
							'_'.$this->multibanco_id.'_ref' => $ref,
							'date_after' => date_i18n( 'Y-m-d', strtotime( '-'.intval( $no_repeat_days ).' days ') ),
						) );
						if ( count($orders) > 0 ) $exists = true;
					}
				}
			}
			if ( $exists ) {
				//Reference exists - Let's try again
				//$seed = intval( $seed ) + 1; //For incremental mode
				$seed = rand( 0,9999 ); //For random mode - Less loop possibility
				$ref = $this->multibanco_create_ref( $ent, $subent, $seed, $total, intval( $no_repeat_days ) );
			}
		} else {
			//No checking - for tests only
		}
		return $ref;
	}
	public function multibanco_create_ref_no_check_digit( $ent, $subent, $id, $total ) {
		$subent = str_pad( intval( $subent ), 3, "0", STR_PAD_LEFT );
		$id = str_pad( intval( $id ), 6, "0", STR_PAD_LEFT );
		$ref = $subent.$id;
		return $ref;
	}

	/* Force Reference creation on New Order (not the British Synthpop band) */
	public function multibanco_woocommerce_process_shop_order_meta_backend() {
		//Workaround for https://wordpress.org/support/topic/referencia-nao-criada-em-encomendas-feitas-manualmente/
		//Backend - This should not be needed when this commit is applied to production https://github.com/woothemes/woocommerce/commit/7dadae7bc80a842e10e78a972334937ed5c4416a
		if ( version_compare( WC_VERSION, '2.6', '<' ) )
			add_action( 'woocommerce_process_shop_order_meta', array( $this, 'multibanco_woocommerce_checkout_update_order_meta' ), 999 );
	}
	public function multibanco_woocommerce_checkout_update_order_meta( $order_id ) {
		$order = new WC_Order_MB_Ifthen( $order_id );
		//Avoid duplicate instructions on the email...
		//remove_action( 'woocommerce_email_before_order_table', array( 'WC_Multibanco_IfThen_Webdados', 'email_instructions_1' ), 10, 3 ); //Not needed since v3 fixes
		if ( $order->mb_get_payment_method() == $this->multibanco_id ) {
			$ref = $this->multibanco_get_ref( $order_id );
			//That should do it...
		}
	}

	/* Change Ref if order total is changed on wp-admin */
	public function multibanco_woocommerce_saved_order_items( $order_id ) {
		if ( is_admin() ) { //Admin?
			//WPML?
			if ( $this->wpml_active ) {
				$this->woocommerce_new_customer_note_fix_wpml_do_it( $order_id );
			}
			$order = new WC_Order_MB_Ifthen( $order_id );
			if ( $order->mb_get_payment_method() == $this->multibanco_id ) { //Multibanco?
				if ( $this->version >= '1.7.9.2' ) {
					//Details already existed - Let's check the order status
					$order_status = $order->mb_get_status();
					if ( in_array( $order_status , array( 'on-hold', 'pending' ) ) ) {
						//Order status OK - Let's check if value has changed
						if (
							( !$order_mb_details = $this->get_multibanco_order_details( $order_id ) )
							||
							( floatval( $order->mb_get_total() ) != floatval( $order_mb_details['val'] ) )
						) {
							$ref = $this->multibanco_get_ref( $order_id, true );
							$this->debug_log( $this->multibanco_id, 'Order '.$order->mb_get_id().' value changed' );
							if ( is_array( $ref ) ) {
								$order->add_order_note(
									sprintf( __( 'The Multibanco payment details have changed', 'multibanco-ifthen-software-gateway-for-woocommerce' ).':
– – – – – – – – – – – – – – – – – - - - -
'.__( 'Previous entity', 'multibanco-ifthen-software-gateway-for-woocommerce' ).': %s
'.__( 'Previous reference', 'multibanco-ifthen-software-gateway-for-woocommerce' ).': %s
'.__( 'Previous value', 'multibanco-ifthen-software-gateway-for-woocommerce' ).': %s
– – – – – – – – – – – – – – – – – - - - -
'.__( 'New entity', 'multibanco-ifthen-software-gateway-for-woocommerce' ).': %s
'.__( 'New reference', 'multibanco-ifthen-software-gateway-for-woocommerce' ).': %s
'.__( 'New value', 'multibanco-ifthen-software-gateway-for-woocommerce' ).': %s
– – – – – – – – – – – – – – – – – - - - -
'.__( 'If the customer pays using the previous details, the payment will be accepted by the Multibanco system, but the order will not be updated via callback.', 'multibanco-ifthen-software-gateway-for-woocommerce' ),
isset( $order_mb_details['ent'] ) ? trim( $order_mb_details['ent'] ) : '',
isset( $order_mb_details['ref'] ) ? $this->format_multibanco_ref( $order_mb_details['ref'] ) : '',
isset( $order_mb_details['val'] ) ? wc_price( $order_mb_details['val'] ) : '',
trim( $ref['ent'] ),
$this->format_multibanco_ref( $ref['ref'] ),
wc_price( $order->mb_get_total() )
									)
							);
							//Notify client?
							if ( $this->multibanco_settings['update_ref_client'] == 'yes' ) {
								WC()->payment_gateways(); //Just in case...
								$order->add_order_note(
									sprintf( __( 'The Multibanco payment details have changed', 'multibanco-ifthen-software-gateway-for-woocommerce' ).':
'.__( 'New entity', 'multibanco-ifthen-software-gateway-for-woocommerce' ).': %s
'.__( 'New reference', 'multibanco-ifthen-software-gateway-for-woocommerce' ).': %s
'.__( 'New value', 'multibanco-ifthen-software-gateway-for-woocommerce' ).': %s',
trim( $ref['ent'] ),
$this->format_multibanco_ref( $ref['ref'] ),
wc_price( $order->mb_get_total() )
										)
										,
										1
									);
								}
								?>
								<script type="text/javascript">
									alert( '<?php _e( 'The Multibanco payment details have changed', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>. <?php echo ( $this->multibanco_settings['update_ref_client'] == 'yes' ? __( 'The customer will be notified' , 'multibanco-ifthen-software-gateway-for-woocommerce' ) : __( 'You should notify the customer' , 'multibanco-ifthen-software-gateway-for-woocommerce' ) ); ?>. <?php _e( 'The page will now reload.' , 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>' );
									location.reload();
								</script>
								<?php
							}
						}
					}
				}
			}
		}
	}

	/* Filter to be able to use wc_get_orders with our Multibanco and MB WAY references */
	public function multibanco_woocommerce_order_data_store_cpt_get_orders_query( $query, $query_vars ) {
		//Multibanco - Entity
		if ( ! empty( $query_vars['_'.$this->multibanco_id.'_ent'] ) ) {
			$query['meta_query'][] = array(
				'key' => '_'.$this->multibanco_id.'_ent',
				'value' => esc_attr( $query_vars['_'.$this->multibanco_id.'_ent'] ),
			);
		}
		//Multibanco - Reference
		if ( ! empty( $query_vars['_'.$this->multibanco_id.'_ref'] ) ) {
			$query['meta_query'][] = array(
				'key' => '_'.$this->multibanco_id.'_ref',
				'value' => esc_attr( $query_vars['_'.$this->multibanco_id.'_ref'] ),
			);
		}
		//MB WAY - Key
		if ( ! empty( $query_vars['_'.$this->mbway_id.'_mbwaykey'] ) ) {
			$query['meta_query'][] = array(
				'key' => '_'.$this->mbway_id.'_mbwaykey',
				'value' => esc_attr( $query_vars['_'.$this->mbway_id.'_mbwaykey'] ),
			);
		}
		//MB WAY - ID Pedido
		if ( ! empty( $query_vars['_'.$this->mbway_id.'_id_pedido'] ) ) {
			$query['meta_query'][] = array(
				'key' => '_'.$this->mbway_id.'_id_pedido',
				'value' => esc_attr( $query_vars['_'.$this->mbway_id.'_id_pedido'] ),
			);
		}
		return $query;
	}

	/* Reduce stock - on 'woocommerce_payment_complete_reduce_order_stock' */
	public function woocommerce_payment_complete_reduce_order_stock( $reduce, $order_id, $payment_method, $stock_when ) {
		if ( $reduce ) {
			$order = new WC_Order_MB_Ifthen( $order_id );
			if ( $order->mb_get_payment_method() == $payment_method ) {
				if ( version_compare( WC_VERSION, '3.4.0', '>=' ) ) { //https://github.com/woocommerce/woocommerce/commit/70c9cff608761fcd48b57f709059e00b1ffeee38#diff-27a48ce67fa604181c90b4bb464164ac
					//After 3.4.0
					if ( $order->mb_has_status( 'on-hold' ) || $order->mb_has_status( 'pending' ) ) {
						//Pending payment
						if ( $stock_when == 'order' ) {
							//Yes, because we want to reduce on the order
							return true;
						} else {
							return false;
						}
					} else {
						//Payment done
						if ( $stock_when == '' ) {
							//Yes, because we want to reduce on payment
							return true;
						} else {
							return false;
						}
					}
				} else {
					//Before 3.4.0 - This only runs for paid orders
					if ( $stock_when == '' ) {
						//Yes, because we want to reduce on payment
						return true;
					} else {
						return false;
					}
				}
			} else {
				return $reduce;
			}
		} else {
			//Already reduced
			return false;
		}
	}


	/* Cancel our orders when WooCommerce cancels pending orders (even if ours are on-hold) */
	/* Cancel unpaid orders - See WooCommerce wc_cancel_unpaid_orders() */
	public function multibanco_woocommerce_cancel_unpaid_orders() {
		$methods = array();
		if ( apply_filters( 'multibanco_ifthen_cancel_unpaid_orders', false ) ) {
			$methods[] = $this->multibanco_id;
		}
		if ( apply_filters( 'mbway_ifthen_cancel_unpaid_orders', false ) ) {
			$methods[] = $this->mbway_id;
		}
		if ( count( $methods ) > 0 ) {
			if ( version_compare( WC_VERSION, '3.0', '<' ) ) return;
			$held_duration = get_option( 'woocommerce_hold_stock_minutes' );
			if ( $held_duration < 1 || 'yes' !== get_option( 'woocommerce_manage_stock' ) ) return;
			$date_before = '-' . absint( $held_duration ) . ' MINUTES';
			foreach ( $methods as $method ) {
				$unpaid_orders = wc_get_orders( array( // https://github.com/woocommerce/woocommerce/wiki/wc_get_orders-and-WC_Order_Query
					'status'			=> array( 'on-hold', 'pending' ),
					'type'				=> array( 'shop_order' ),
					'limit'				=> -1,
					'date_modified'		=> '<' . strtotime( $date_before ),
					'payment_method'	=> $method,
				) );
				if ( $unpaid_orders ) {
					foreach ( $unpaid_orders as $unpaid_order ) {
						if ( apply_filters( 'woocommerce_cancel_unpaid_order', 'checkout' === $unpaid_order->get_created_via(), $unpaid_order ) ) {
							$unpaid_order->update_status( 'cancelled', __( 'Unpaid order cancelled - time limit exceeded.', 'woocommerce' ) );
							// Restore stock levels
							switch( $method ) {
								case $this->multibanco_id:
									$filter_stock = 'multibanco_ifthen_cancel_unpaid_orders_restore_stock';
									$action = 'multibanco_ifthen_unpaid_order_cancelled';
									break;
								case $this->mbway_id:
									$filter_stock = 'mbway_ifthen_cancel_unpaid_orders_restore_stock';
									$action = 'mbway_ifthen_unpaid_order_cancelled';
									break;
							}
							if ( apply_filters( $filter_stock, false, $unpaid_order->get_id() ) && $unpaid_order->get_data_store()->get_stock_reduced( $unpaid_order->get_id() ) ) {
								foreach ( $unpaid_order->get_items() as $item_id => $item ) {
									// Get an instance of corresponding the WC_Product object
									if ( $product = $item->get_product() ) {
										$qty = $item->get_quantity(); // Get the item quantity
										wc_update_product_stock( $product, $qty, 'increase' );
									}
								}
							}
							do_action( $action, $unpaid_order->get_id() );
						}
					}
				}
			}
		}
	}

	/* SMS instructions - General. Can be used to feed any SMS gateway/plugin */
	public function multibanco_sms_instructions( $message, $order_id ) {
		$order = new WC_Order_MB_Ifthen( $order_id );
		$instructions = ''; //We return an empty string so that we always replace our placeholder, even if it's not our gateway
		if ( $order->mb_get_payment_method() == $this->multibanco_id ) {
			switch ( $order->mb_get_status() ) {
			case 'on-hold':
			case 'pending':
				$ref = $this->multibanco_get_ref( $order_id );
				if ( is_array( $ref) ) {
					$instructions = 
						__( 'Ent', 'multibanco-ifthen-software-gateway-for-woocommerce' )
						.' '
						.$ref['ent']
						.' '
						.__( 'Ref', 'multibanco-ifthen-software-gateway-for-woocommerce' )
						.' '.$this->format_multibanco_ref( $ref['ref'] )
						.' '
						.__( 'Val', 'multibanco-ifthen-software-gateway-for-woocommerce' )
						.' '
						.$order->mb_get_total();
					$instructions = preg_replace( '/\s+/', ' ', $instructions ); //Remove extra spaces if necessary
					//Filters in case the website owner wants to customize the message
					$instructions = apply_filters( 'multibanco_ifthen_sms_instructions', $instructions, $ref['ent'], $ref['ref'], $order->mb_get_total(), $order_id );
				} else {
					//error getting ref
				}
				break;
			case 'processing':
				//No instructions
				break;
			default:
				return;
				break;
			}
		}
		return $instructions;
	}

	/* APG SMS integration (Removed from the class because now APG also sends scheduled SMS and the payment class may not be initiated) */
	public function multibanco_apg_sms_message( $message, $order_id ) {
		$replace = $this->multibanco_sms_instructions( $message, $order_id ); //Get instructions
		return trim( str_replace( '%multibanco_ifthen%', $replace, $message ) ); //Return message with %multibanco_ifthen% replaced by the instructions
	}


	/* WooCommece Subscriptions - Do not copy our fields for renewal and resubscribe orders */
	public function multibanco_wcs_filter_meta( $meta ) {
		$mb_fields = array(
			'_'.$this->multibanco_id.'_ent',
			'_'.$this->multibanco_id.'_ref',
			'_'.$this->multibanco_id.'_val',
		);
		foreach ( $meta as $key => $value ) {
			if ( isset( $value['meta_key'] ) && in_array( $value['meta_key'], $mb_fields ) ) {
				unset( $meta[$key] );
			}
		}
		return $meta;
	}
	/* WooCommerce Subscriptions - Set renewal order on hold */
	public function multibanco_wcs_renewal_order_created( $renewal_order, $subscription ) {
		//if ( version_compare( WC_VERSION, '3.0', '>=' ) ) {
			if ( ! is_object( $subscription ) ) {
				$subscription = wcs_get_subscription( $subscription );
			}
			if ( ! is_object( $renewal_order ) ) {
				$renewal_order = wc_get_order( $renewal_order );
			}
			if ( is_a( $renewal_order, 'WC_Order' ) && wcs_is_subscription( $subscription ) ) {
				$subscription_payment_method = version_compare( WC_VERSION, '3.0', '>=' ) ? $subscription->get_payment_method() : $subscription->payment_method;
				if ( $subscription_payment_method == $this->multibanco_id ) { //Subscription was inially paid by Multibanco?
					if ( $this->multibanco_settings['support_woocommerce_subscriptions'] == 'yes' ) {
						//Set payment method
						$renewal_order->set_payment_method( 'WC_Multibanco_IfThen_Webdados' );
						//Forces MB Ref creation
						$renewal_order_id = version_compare( WC_VERSION, '3.0', '>=' ) ? $renewal_order->get_id() : $renewal_order->id;
						$ref = $this->multibanco_get_ref( $renewal_order_id, true );
						if ( is_array( $ref) ) {
							//Changes to "on hold" - Forces email sending
							$renewal_order->update_status( 'on-hold', __( 'Awaiting Multibanco payment.', 'multibanco-ifthen-software-gateway-for-woocommerce' ).' (WooCommerce Subscriptions)' );
						}
					}
				}
			}
		//}
		return $renewal_order;
	}

	/* Set email correct language - Stolen from WCML emails.class.php - Not sure if this is still needed */
	public function change_email_language( $lang ) {
		global $sitepress;
		//Unload
		unload_textdomain( 'multibanco-ifthen-software-gateway-for-woocommerce' );
		if ( $lang=='en' ) {
			//English? Just use plugin default strings
		} else {
			$this->locale = $sitepress->get_locale( $lang );
			add_filter( 'plugin_locale', array( $this, 'set_locale_for_emails' ), 10, 2 );
			load_plugin_textdomain( 'multibanco-ifthen-software-gateway-for-woocommerce' );
			remove_filter( 'plugin_locale', array( $this, 'set_locale_for_emails' ), 10, 2 );
		}
	}
	public function set_locale_for_emails( $locale, $domain ) {
		if ( $domain == 'multibanco-ifthen-software-gateway-for-woocommerce' && $this->locale ) {
			$locale = $this->locale;
		}
		return $locale;
	}

	/* WooCommerce 3.0 is not allowing payment gateways to add information to transactional emails - Let's fix it for everybody, shall we? */
	/* https://github.com/woocommerce/woocommerce/issues/13966 - Should be fixed now */
	public function woocommerce_send_queued_transactional_email( $filter = '', $args = array() ) {
		//Only in 3.0.0 - It should be fixed on other versions (we hope)
		if ( version_compare( WC_VERSION, '3.0.0', '==' ) ) {
			WC()->payment_gateways();
			WC()->shipping();
		}
	}

	/* WPML AJAX fix locale */
	public function wpml_ajax_fix_locale() {
		//If WPML is present and we're loading via ajax, let's try to fix the locale
		if ( $this->wpml_active ) {
			if ( function_exists( 'wpml_is_ajax' ) && wpml_is_ajax() ) {  //We check the function because we may be using Polylang
				if ( ICL_LANGUAGE_CODE != 'en' ) {
					add_filter( 'plugin_locale', array( $this, 'wpml_ajax_fix_locale_do_it' ), 1, 2 );
				}
			}
		}
	}
	/* This should NOT be needed! - Check with WooCommerce Multilingual team */
	public function wpml_ajax_fix_locale_do_it( $locale, $domain ) {
		if ( $domain == 'multibanco-ifthen-software-gateway-for-woocommerce' ) {
			global $sitepress;
			$locales = icl_get_languages_locales();
			if ( isset( $locales[ICL_LANGUAGE_CODE] ) ) $locale = $locales[ICL_LANGUAGE_CODE];
			//But if it's notes
			if ( $this->mb_ifthen_locale ) $locale = $this->mb_ifthen_locale;
		}
		return $locale;
	}
	/* Languages on Notes emails - We need to check if it's our order (Multibanco or MBWay) */
	public function woocommerce_new_customer_note_fix_wpml( $order_id ) {
		if ( is_array( $order_id ) ){
			if ( isset( $order_id['order_id'] ) ) {
				$order_id = $order_id['order_id'];
			} else {
				return;
			}
		}
		if ( $this->wpml_active ) {
			$this->woocommerce_new_customer_note_fix_wpml_do_it( $order_id );
		}
	}
	public function woocommerce_new_customer_note_fix_wpml_do_it( $order_id ) {
		global $sitepress;
		$order = new WC_Order_MB_Ifthen( $order_id );
		$lang = $order->mb_get_wpml_language();
		if( !empty( $lang ) && $lang != $sitepress->get_default_language() ){
			$this->mb_ifthen_locale = $sitepress->get_locale( $lang ); //Set global to be used on wpml_ajax_fix_locale_do_it above
			add_filter( 'plugin_locale', array( $this, 'wpml_ajax_fix_locale_do_it' ), 1, 2 );
			mbifthen_load_textdomain();
		}
	}

	/* Right sidebar on payment gateway settings */
	public function admin_right_bar() {
		?>
		<div id="wc_ifthen_rightbar">
			<h4><?php _e( 'Commercial information', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>:</h4>
			<p><a href="https://ifthenpay.com/<?php echo esc_attr( $this->out_link_utm); ?>" title="<?php echo esc_attr( sprintf( __( 'Please contact %s', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 'IfthenPay' ) ); ?>" target="_blank"><img src="<?php echo plugins_url( 'images/ifthenpay.png', __FILE__ ); ?>" width="200"/></a></p>
			<h4><?php _e( 'Technical support or custom WordPress/WooCommerce development', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>:</h4>
			<p><a href="https://www.webdados.pt/contactos/<?php echo esc_attr( $this->out_link_utm); ?>" title="<?php echo esc_attr( sprintf( __( 'Please contact %s', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 'Webdados' ) ); ?>" target="_blank"><img src="<?php echo plugins_url( 'images/webdados.svg', __FILE__ ); ?>" width="200"/></a></p>
			<h4><?php _e( 'Please rate our plugin at WordPress.org', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>:</h4>
			<a href="https://wordpress.org/support/view/plugin-reviews/multibanco-ifthen-software-gateway-for-woocommerce?filter=5#postform" target="_blank" style="text-align: center; display: block;">
				<div class="star-rating"><div class="star star-full"></div><div class="star star-full"></div><div class="star star-full"></div><div class="star star-full"></div><div class="star star-full"></div></div>
			</a>
			<hr/>
			<iframe src="https://www.webdados.pt/multibancoiframe" scrolling="no"></iframe>
			<div class="clear"></div>
		</div>
		<?php
	}

	/* MB WAY Ajax order status */
	public function mbway_ajax_order_status() {
		$order_id = wc_get_order_id_by_order_key( trim( $_POST['order_key'] ) );
		if ( intval( $order_id ) > 0 && intval( $_POST['order_id'] ) == intval( $order_id ) ) {
			$order = new WC_Order_MB_Ifthen( intval( $order_id ) );
			echo json_encode(
				array( 'order_status' => $order->mb_get_status() )
			);
		} else {
			echo json_encode(
				array( 'order_status' => '' )
			);
		}
		die();
	}

	/* MB WAY - Request payment again */
	public function wp_ajax_mbway_ifthen_request_payment_again() {
		if ( wp_verify_nonce( $_REQUEST['nonce'], 'mbway_ifthen_request_payment_again') ) {
			if ( isset( $_REQUEST['order_id'] ) && intval( $_REQUEST['order_id'] ) > 0 && isset( $_REQUEST['order_id'] ) ) {
				$order = new WC_Order_MB_Ifthen( intval( $_REQUEST['order_id'] ) );
				$mbway = new WC_MBWAY_IfThen_Webdados;
				$phone = isset( $_REQUEST['phone'] ) ? trim( sanitize_text_field( $_REQUEST['phone'] ) ) : '';
				if ( $mbway->webservice_set_pedido( $order->mb_get_id(), $phone ) ) {
					echo json_encode( array(
						'status' => 1,
						'message'  => __( 'MB WAY Payment has been requested', 'multibanco-ifthen-software-gateway-for-woocommerce' )
					) );
				} else {
					echo json_encode( array(
						'status' => 0,
						'error'  => __( 'Error contacting IfthenPay servers to create MB WAY Payment', 'multibanco-ifthen-software-gateway-for-woocommerce' )
					) );
				}
			} else {
				echo json_encode( array(
					'status' => 0,
					'error'  => __( 'Invalid parameters', 'mbway_ifthen_request_payment_again' )
				) );
			}
		} else {
			echo json_encode( array(
				'status' => 0,
				'error'  => __( 'Error', 'mbway_ifthen_request_payment_again' )
			) );
		}
		wp_die();
	}

}