<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

define( 'MBWAY_IFTHEN_DESC_LEN', 70 );

/**
 * Multibanco IfThen Class.
 *
 */
if ( ! class_exists( 'WC_MBWAY_IfThen_Webdados' ) ) {

	class WC_MBWAY_IfThen_Webdados extends WC_Payment_Gateway {

		/* Single instance */
		protected static $_instance = null;
		public static $instances = 0;
		
		/**
		 * Constructor for your payment class
		 *
		 * @access public
		 * @return void
		 */
		public function __construct() {

			self::$instances++;

			$this->id = WC_IfthenPay_Webdados()->mbway_id;
	
			// Logs
			$this->debug = ( $this->get_option( 'debug' )=='yes' ? true : false );
			$this->debug_email = $this->get_option( 'debug_email' );
			
			//Check version and upgrade
			$this->version = WC_IfthenPay_Webdados()->version;
			$this->upgrade();
	
			$this->icon = ( $this->get_option( 'small_icon' ) == 'yes' ? plugins_url( 'images/icon_mbway_48_small.png', __FILE__ ) : plugins_url( 'images/icon_mbway_48.png', __FILE__ ) );
			$this->has_fields = true;

			$this->method_title = __( 'Pagamento MB WAY no telemóvel (IfthenPay)', 'multibanco-ifthen-software-gateway-for-woocommerce' );
			$this->method_description = __( 'Easy and simple payment using "MB WAY" on your mobile phone. (Only available for customers of Portuguese banks with MB WAY app installed. Payment service provided by IfthenPay.)', 'multibanco-ifthen-software-gateway-for-woocommerce' );
			if ( $this->get_option( 'support_woocommerce_subscriptions' ) == 'yes' ) {
				$this->supports = array( 'products', 'subscriptions', 'subscription_suspension', 'subscription_reactivation', 'subscription_date_changes' ); //products is by default
			}
			$this->secret_key = $this->get_option( 'secret_key' );
			if ( trim( $this->secret_key )=='' ) {
				//First load?
				$this->secret_key = md5( home_url().time().rand(0,999) );
				//Let's set the callback activation email as NOT sent
				update_option( $this->id . '_callback_email_sent', 'no' );
			}

			//Webservice
			$this->webservice_url = 'https://www.ifthenpay.com/mbwayWS/IfthenPayMBW.asmx';

			//on hold or pending?
			$this->order_initial_status_pending = apply_filters( 'mbway_ifthen_order_initial_status_pending', true );
	
			//Plugin options and settings
			$this->init_form_fields();
			$this->init_settings();
	
			//User settings
			$this->title = $this->get_option( 'title' );
			$this->description = $this->get_option( 'description' );
			$this->extra_instructions = $this->get_option( 'extra_instructions' );
			$this->mbwaykey = $this->get_option( 'mbwaykey' );
			$this->settings_saved = $this->get_option( 'settings_saved' );
			$this->send_to_admin = ( $this->get_option( 'send_to_admin' ) == 'yes' ? true : false );
			$this->only_portugal = ( $this->get_option( 'only_portugal' ) == 'yes' ? true : false );
			$this->only_above = $this->get_option( 'only_above' );
			$this->only_bellow = $this->get_option( 'only_bellow' );
			$this->stock_when = $this->get_option( 'stock_when' );
	 	
			// Actions and filters
			if ( self::$instances == 1 ) { //Avoid duplicate actions and filters if it's initiated more than once (if WooCommerce loads after us)
				add_action( 'woocommerce_update_options_payment_gateways_'.$this->id, array( $this, 'process_admin_options' ) );
				add_action( 'woocommerce_update_options_payment_gateways_'.$this->id, array( $this, 'send_callback_email' ) );
				if ( WC_IfthenPay_Webdados()->wpml_active ) add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'register_wpml_strings' ) );
				add_action( 'woocommerce_thankyou_'.$this->id, array( $this, 'thankyou' ) );
				add_action( 'woocommerce_order_details_after_order_table', array( $this, 'order_details_after_order_table' ), 9 );
				add_filter( 'woocommerce_available_payment_gateways', array( $this, 'disable_if_currency_not_euro' ) );
				add_filter( 'woocommerce_available_payment_gateways', array( $this, 'disable_unless_portugal' ) );
				add_filter( 'woocommerce_available_payment_gateways', array( $this, 'disable_only_above_or_bellow' ) );

				// NO SMS Integrations for MB WAY
		 		
				// Customer Emails
				//add_action( 'woocommerce_email_before_order_table', array( $this, 'email_instructions' ), 10, 3 ); - "Hyyan WooCommerce Polylang Integration" removes this action
				add_action( 'woocommerce_email_before_order_table', array( $this, 'email_instructions_1' ), 10, 3 ); //Avoid "Hyyan WooCommerce Polylang Integration" remove_action
		
				// Payment listener/API hook
				add_action( 'woocommerce_api_wc_mbway_ifthen_webdados', array( $this, 'callback' ) );
		
				// Filter to decide if payment_complete reduces stock, or not
				add_filter( 'woocommerce_payment_complete_reduce_order_stock', array( $this, 'woocommerce_payment_complete_reduce_order_stock' ), 10, 2 );
		
				// Admin notice if callback activation email is still not sent
				add_action( 'admin_notices', array( $this, 'admin_notices' ) );

			}

			// Ensures only one instance of our plugin is loaded or can be loaded - works if WooCommerce loads the payment gateways before we do
			if ( is_null( self::$_instance ) ) {
				self::$_instance = $this;
			}
			
		}

		/* Ensures only one instance of our plugin is loaded or can be loaded */
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}

		/**
		 * Upgrades (if needed)
		 */
		function upgrade() {
			if ( $this->get_option( 'version' ) < $this->version ) {
				//Upgrade
				$this->debug_log( 'Upgrade to '.$this->version.' started' );
				//Nothing so far
				//...
				//Upgrade on the database - Risky?
				$temp = get_option( 'woocommerce_'.$this->id.'_settings', '' );
				if ( !is_array($temp) ) $temp = array();
				$temp['version'] = $this->version;
				update_option( 'woocommerce_'.$this->id.'_settings', $temp );
				$this->debug_log( 'Upgrade to '.$this->version.' finished' );
			}
		}

		/**
		 * WPML compatibility
		 */
		function register_wpml_strings() {
			//These are already registered by WooCommerce Multilingual
			/*$to_register=array(
				'title',
				'description',
			);*/
			$to_register = array(
				'extra_instructions'
			);
			foreach( $to_register as $string ) {
				icl_register_string( $this->id, $this->id.'_'.$string, $this->settings[$string] );
			}
		}

		/**
		 * Initialise Gateway Settings Form Fields
		 * 'setting-name' => array(
		 *		'title' => __( 'Title for setting', 'woothemes' ),
		 *		'type' => 'checkbox|text|textarea',
		 *		'label' => __( 'Label for checkbox setting', 'woothemes' ),
		 *		'description' => __( 'Description for setting' ),
		 *		'default' => 'default value'
		 *	),
		 */
		function init_form_fields() {
		
			$this->form_fields = array(
				'enabled' => array(
								'title' => __( 'Enable/Disable', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
								'type' => 'checkbox', 
								'label' => __( 'Enable "MB WAY" (using IfthenPay)', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
								'default' => 'no'
							),
				'mbwaykey' => array(
								'title' => __( 'MB WAY Key', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
								'type' => 'input',
								'description' => __( 'MB WAY Key provided by IfthenPay when signing the contract.', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
								'default' => '',
								'css' => 'width: 130px;',
								'placeholder' => 'XXX-000000',
								'custom_attributes' => array(
									'maxlength'	=> 10,
									'size' => 14
								)
							),
			);
			//if ( strlen( trim( $this->get_option( 'mbwaykey' ) ) ) == 10 && trim( $this->secret_key ) != '' ) {
				$this->form_fields = array_merge( $this->form_fields, array(
					'secret_key' => array(
									'title' => __( 'Anti-phishing key', 'multibanco-ifthen-software-gateway-for-woocommerce' ).' ('.__( 'for MB WAY', 'multibanco-ifthen-software-gateway-for-woocommerce' ).')', 
									'type' => 'hidden', 
									'description' => '<strong id="woocommerce_'.$this->id.'_secret_key_label">'.$this->secret_key.'</strong><br/>'.__( 'To ensure callback security, generated by the system and which must be provided to IfthenPay when asking for the callback activation.', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
									'default' => $this->secret_key 
								),
					'title' => array(
									'title' => __( 'Title', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
									'type' => 'text', 
									'description' => __( 'This controls the title which the user sees during checkout.', 'multibanco-ifthen-software-gateway-for-woocommerce' )
													.( WC_IfthenPay_Webdados()->wpml_active ? ' '.__( 'You should translate this string in <a href="admin.php?page=wpml-string-translation%2Fmenu%2Fstring-translation.php">WPML - String Translation</a> after saving the settings', 'multibanco-ifthen-software-gateway-for-woocommerce' ) : '' ), 
									'default' => __( 'MB WAY', 'multibanco-ifthen-software-gateway-for-woocommerce' )
								),
					'description' => array(
									'title' => __( 'Description', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
									'type' => 'textarea',
									'description' => __( 'This controls the description which the user sees during checkout.', 'multibanco-ifthen-software-gateway-for-woocommerce' )
													.( WC_IfthenPay_Webdados()->wpml_active ? ' '.__( 'You should translate this string in <a href="admin.php?page=wpml-string-translation%2Fmenu%2Fstring-translation.php">WPML - String Translation</a> after saving the settings', 'multibanco-ifthen-software-gateway-for-woocommerce' ) : '' ), 
									'default' => version_compare( WC_VERSION, '2.6', '>=' ) ? $this->get_method_description() : $this->method_description
								),
					'small_icon' => array(
									'title' => __( 'Use small icon?', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
									'type' => 'checkbox', 
									'label' => __( 'Use a small MB WAY icon on the checkout', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
									'default' => 'yes'
								),
					'extra_instructions' => array(
									'title' => __( 'Extra instructions', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
									'type' => 'textarea',
									'description' => __( 'This controls the text which the user sees below the payment details on the "Thank you" page and "New order" email.', 'multibanco-ifthen-software-gateway-for-woocommerce' )
													.( WC_IfthenPay_Webdados()->wpml_active ? ' '.__( 'You should translate this string in <a href="admin.php?page=wpml-string-translation%2Fmenu%2Fstring-translation.php">WPML - String Translation</a> after saving the settings', 'multibanco-ifthen-software-gateway-for-woocommerce' ) : '' ), 
									'default' => __( 'Use the MB WAY app on your phone to approve the payment.', 'multibanco-ifthen-software-gateway-for-woocommerce' )
								),
					'only_portugal' => array(
									'title' => __( 'Only for Portuguese customers?', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
									'type' => 'checkbox', 
									'label' => __( 'Enable only for customers whose address is in Portugal', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
									'default' => 'no'
								),
					'only_above' => array(
									'title' => __( 'Only for orders above', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
									'type' => 'number', 
									'description' => __( 'Enable only for orders above x &euro;. Leave blank (or zero) to allow for any order value.', 'multibanco-ifthen-software-gateway-for-woocommerce' ).' <br/> '.__( 'By design, Multibanco only allows payments from 1 to 999999 &euro; (inclusive). You can use this option to further limit this range.', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
									'default' => ''
								),
					'only_bellow' => array(
									'title' => __( 'Only for orders below', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
									'type' => 'number', 
									'description' => __( 'Enable only for orders below x &euro; (exclusive). Leave blank (or zero) to allow for any order value.', 'multibanco-ifthen-software-gateway-for-woocommerce' ).' <br/> '.__( 'By design, Multibanco only allows payments from 1 to 999999 &euro; (inclusive). You can use this option to further limit this range.', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
									'default' => ''
								),
				) );
				if ( ! $this->order_initial_status_pending ) {
					$this->form_fields = array_merge( $this->form_fields, array(
						'stock_when' => array(
										'title' => __( 'Reduce stock', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
										'type' => 'select', 
										'description' => __( 'Choose when to reduce stock.', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
										'default' => '',
										'options'	=> array(
											'order'	=> __( 'when order is placed (before payment, WooCommerce default)', 'multibanco-ifthen-software-gateway-for-woocommerce' ),
											''		=> __( 'when order is paid (requires active callback)', 'multibanco-ifthen-software-gateway-for-woocommerce' ),
										),
									),
					) );
				}
				$this->form_fields = array_merge( $this->form_fields, array(
					'support_woocommerce_subscriptions' => array(
									'title' => __( 'WooCommerce Subscriptions', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
									'type' => 'checkbox', 
									'label' => __( 'Enable WooCommerce Subscriptions (experimental) support.', 'multibanco-ifthen-software-gateway-for-woocommerce' ),
									'description' => __( 'Shows "MB WAY" (using IfthenPay) as a supported payment gateway, and automatically sets subscription renewal orders to be paid with MB WAY if the original subscription used this payment method. If this option is not activated, MB WAY will only be available as a payment method for subscriptions if the "Manual Renewal Payments" option is enabled on WooCommerce Subscriptions settings.', 'multibanco-ifthen-software-gateway-for-woocommerce' ),
									'default' => 'no'
								),
					'send_to_admin' => array(
									'title' => __( 'Send instructions to admin?', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
									'type' => 'checkbox', 
									'label' => __( 'Should the payment details also be sent to admin?', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
									'default' => 'yes'
								),
					/*'update_ref_client' => array(
									'title' => __( 'Email reference update to client? - THIS SHOULD GENERATE A NEW WEBSERVICE CALL', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
									'type' => 'checkbox', 
									'label' => __( 'If the payment details change because of an update on the backend, should the client be notified?', 'multibanco-ifthen-software-gateway-for-woocommerce' ), 
									'default' => 'no'
								),*/
					'debug' => array(
									'title' => __( 'Debug Log', 'multibanco-ifthen-software-gateway-for-woocommerce' ),
									'type' => 'checkbox',
									'label' => __( 'Enable logging', 'multibanco-ifthen-software-gateway-for-woocommerce' ),
									'default' => 'no',
									'description' => sprintf(
														__( 'Log plugin events, such as callback requests, in %s', 'multibanco-ifthen-software-gateway-for-woocommerce' ),
														( version_compare( WC_VERSION, '3.0', '>=' ) && defined( 'WC_LOG_HANDLER' ) && 'WC_Log_Handler_DB' === WC_LOG_HANDLER )
														?
														'<a href="admin.php?page=wc-status&tab=logs&source='.esc_attr( $this->id ).'" target="_blank">'.__( 'WooCommerce &gt; Status &gt; Logs', 'multibanco-ifthen-software-gateway-for-woocommerce' ).'</a>'
														:
														'<code>'.wc_get_log_file_path( $this->id ).'</code>'
													),
								),
					'debug_email' => array(
									'title' => __( 'Debug to email', 'multibanco-ifthen-software-gateway-for-woocommerce' ),
									'type' => 'email',
									'label' => __( 'Enable email logging', 'multibanco-ifthen-software-gateway-for-woocommerce' ),
									'default' => '',
									'description' => __( 'Send plugin events to this email address, such as callback requests.', 'multibanco-ifthen-software-gateway-for-woocommerce' ),
								)
				) );
			//}
			$this->form_fields = array_merge( $this->form_fields ,array(
				'settings_saved' => array(
								'title' => '', 
								'type' => 'hidden',
								'default' => 0
							),
			) );

			//Allow other plugins to add settings fields
			$this->form_fields = array_merge( $this->form_fields , apply_filters( 'multibanco_ifthen_mbway_settings_fields', array( ) ) );
		
		}
		public function admin_options() {
			?>
			<div id="wc_ifthen">
				<?php if ( ! apply_filters( 'multibanco_ifthen_hide_settings_right_bar', false ) ) WC_IfthenPay_Webdados()->admin_right_bar(); ?>
				<div id="wc_ifthen_settings">
					<h2><?php echo esc_html( version_compare( WC_VERSION, '2.6', '>=' ) ? $this->get_method_title() : $this->method_title ); ?> <small>v.<?php echo $this->version; ?></small></h2>
					<?php echo wp_kses_post( wpautop( version_compare( WC_VERSION, '2.6', '>=' ) ? $this->get_method_description() : $this->method_description ) ); ?>
					<p><strong><?php _e( 'In order to use this plugin you <u>must</u>:', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?></strong></p>
					<ul class="wc_ifthen_list">
						<li><?php printf( __( 'Set WooCommerce currency to <strong>Euros (&euro;)</strong> %1$s', 'multibanco-ifthen-software-gateway-for-woocommerce' ), '<a href="admin.php?page=wc-settings&amp;tab=general">&gt;&gt;</a>.' ); ?></li>
						<li><?php printf( __( 'Sign a contract with %1$s. To know more about this service, please go to %2$s.', 'multibanco-ifthen-software-gateway-for-woocommerce' ), '<strong><a href="https://ifthenpay.com/'.esc_attr( WC_IfthenPay_Webdados()->out_link_utm ).'" target="_blank">IfthenPay</a></strong>', '<a href="https://ifthenpay.com/'.esc_attr( WC_IfthenPay_Webdados()->out_link_utm ).'" target="_blank">https://ifthenpay.com</a>' ); ?></li>
						<li><?php _e( 'Fill out all the details (MB Way Key) provided by <strong>IfthenPay</strong> in the fields below.', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>
						<li class="mb_hide_extra_fields"><?php printf( __( 'Ask IfthenPay to activate "MB WAY Callback" on your account using this exact URL: %1$s and this Anti-phishing key: %2$s', 'multibanco-ifthen-software-gateway-for-woocommerce' ), '<br/><code><strong>'.WC_IfthenPay_Webdados()->mbway_notify_url.'</strong></code><br/>', '<br/><code><strong>'.$this->secret_key.'</strong></code>' ); ?></li>
					</ul>
					<?php
					$hide_extra_fields = false;
					if(
						trim( strlen( $this->mbwaykey) ) == 10
						&&
						trim( $this->secret_key ) != ''
					) {
						if ( $callback_email_sent = get_option( $this->id . '_callback_email_sent' ) ) { //No notice for older versions
							if ( $callback_email_sent == 'no' ) {
								if ( !isset( $_GET['callback_warning'] ) ) {
									?>
									<div id="message" class="error">
										<p><strong><?php _e( 'You haven\'t yet asked IfthenPay for the "Callback" activation. The orders will NOT be automatically updated upon payment.', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?></strong></p>
									</div>
									<?php
								}
							}
						}
						?>
						<p id="wc_ifthen_callback_open_p"><a href="#" id="wc_ifthen_callback_open" class="button button-small"><?php _e( 'Click here to ask IfthenPay to activate the "Callback"', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?></a></p>
						<div id="wc_ifthen_callback_div">
							<p><?php _e( 'This will submit a request to IfthenPay, asking them to activate the "Callback" on your account. If you have already asked for the "Callback" activation, wait for their feedback before submiting a new request. The following details will be sent via email to IfthenPay (with CC to your email address):', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?></p>
							<table class="form-table">
								<tr valign="top">
									<th scope="row" class="titledesc"><?php _e( 'Email', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?></th>
									<td class="forminp">
										<?php echo get_option( 'admin_email' ); ?>
									</td>
								</tr>
								<tr valign="top">
									<th scope="row" class="titledesc">MB WAY Key</th>
									<td class="forminp">
										<?php echo $this->mbwaykey; ?>
									</td>
								</tr>
								<tr valign="top">
									<th scope="row" class="titledesc"><?php _e( 'Anti-phishing key', 'multibanco-ifthen-software-gateway-for-woocommerce' ).' ('.__( 'for MB WAY', 'multibanco-ifthen-software-gateway-for-woocommerce' ).')'; ?></th>
									<td class="forminp">
										<?php echo $this->secret_key; ?>
									</td>
								</tr>
								<tr valign="top">
									<th scope="row" class="titledesc"><?php _e( 'Callback URL', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?></th>
									<td class="forminp">
										<?php echo WC_IfthenPay_Webdados()->mbway_notify_url; ?>
									</td>
								</tr>
							</table>
							<p style="text-align: center;">
								<strong><?php _e( 'Attention: if you ever change from HTTP to HTTPS or vice versa, or the permalinks structure,<br/>you may have to ask IfthenPay to update the callback URL.', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?></strong>
							</p>
							<p style="text-align: center; margin-bottom: 0px;">
								<input type="hidden" id="wc_ifthen_callback_send" name="wc_ifthen_callback_send" value="0"/>
								<input id="wc_ifthen_callback_submit" class="button-primary" type="button" value="<?php _e( 'Ask for Callback activation', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>"/>
								<input id="wc_ifthen_callback_cancel" class="button" type="button" value="<?php _e( 'Cancel', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>"/>
							</p>
						</div>
						<?php
					} else {
						$hide_extra_fields = true;
						if ( $this->settings_saved == 1 ) {
							?>
							<div id="message" class="error">
								<p><strong><?php _e( 'Invalid MB WAY Key (exactly 10 characters).', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?></strong></p>
							</div>
							<?php
						} else {
							?>
							<div id="message" class="error">
								<p><strong><?php _e( 'Set the MB WAY Key and Save changes to set other plugin options.', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?></strong></p>
							</div>
							<?php
						}
					}
					?>
					<hr/>
					<script type="text/javascript">
					jQuery( document ).ready( function() {
						<?php if ( $hide_extra_fields ) { ?>
							//Hide extra fields if there are errors on MB WAY Key
							jQuery( '#wc_ifthen_settings table.form-table tr:nth-child(n+3)' ).hide();
							jQuery( '#wc_ifthen_settings .mb_hide_extra_fields' ).hide();
						<?php } ?>
						//Save secret key
						if (jQuery( '#woocommerce_<?php echo $this->id; ?>_secret_key' ).val()=='' ) {
							jQuery( '#woocommerce_<?php echo $this->id; ?>_secret_key' ).val( '<?php echo $this->secret_key; ?>' );
							jQuery( '#woocommerce_<?php echo $this->id; ?>_secret_key_label' ).html( '<?php echo $this->secret_key; ?>' );
							jQuery( '#mainform' ).submit();
						}
						//Settings saved
						jQuery( '#woocommerce_<?php echo $this->id; ?>_settings_saved' ).val( '1' );
						//Callback activation
						jQuery( '#wc_ifthen_callback_open' ).click( function() {
							ifthen_callback_open();
							return false;
						});
						jQuery( '#wc_ifthen_callback_cancel' ).click( function() {
							jQuery( '#wc_ifthen_callback_div' ).toggle();
							jQuery( '#wc_ifthen_callback_open_p' ).toggle();
							return false;
						});
						//Callback send
						jQuery( '#wc_ifthen_callback_submit' ).click( function() {
							if ( confirm( '<?php echo strip_tags( __( 'Are you sure you want to ask IfthenPay to activate the "Callback"?', 'multibanco-ifthen-software-gateway-for-woocommerce' ) ); ?>' ) ) {
								jQuery( '#wc_ifthen_callback_send' ).val( 1 );
								jQuery( '#mainform' ).submit();
								return true;
							} else {
								return false;
							}
						});
						function ifthen_callback_open() {
							jQuery( '#wc_ifthen_callback_div' ).toggle();
							jQuery( '#wc_ifthen_callback_open_p' ).toggle();
						}
						<?php
						if ( $callback_email_sent=get_option( $this->id . '_callback_email_sent' ) ) {
							if ( $callback_email_sent=='no' ) { 
								?>
								jQuery( '#wc_ifthen_callback_open' ).addClass('button-link-delete');
								ifthen_callback_open();
								<?php
								if ( isset( $_GET['callback_warning'] ) && intval( $_GET['callback_warning'] ) == 1 ) {
									?>
									jQuery( '#wc_ifthen_callback_div' ).addClass('focus' );
									setTimeout( function() {
										jQuery( '#wc_ifthen_callback_div' ).removeClass('focus' );
									}, 1000 );
									<?php
								}
							}
						}
						?>
					});
					</script>
					<table class="form-table">
					<?php
					if ( trim( get_woocommerce_currency() ) == 'EUR' ) {
						$this->generate_settings_html();
					} else {
						?>
						<p><strong><?php _e( 'ERROR!', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?> <?php printf( __( 'Set WooCommerce currency to <strong>Euros (&euro;)</strong> %1$s', 'multibanco-ifthen-software-gateway-for-woocommerce' ), '<a href="admin.php?page=wc-settings&amp;tab=general">'.__( 'here', 'multibanco-ifthen-software-gateway-for-woocommerce' ).'</a>.' ); ?></strong></p>
						<?php
					}
					?>
					</table>
				</div>
			</div>
			<div class="clear"></div>
			<style type="text/css">
				#wc_ifthen_rightbar {
					display: none;
				}
				@media (min-width: 961px) {
					#wc_ifthen {
						height: auto;
						overflow: hidden;
					}
					#wc_ifthen_settings {
						width: auto;
						overflow: hidden;
					}
					#wc_ifthen_rightbar {
						display: block;
						float: right;
						width: 220px;
						max-width: 20%;
						margin-left: 20px;
						padding: 15px;
						background-color: #fff;
					}
					#wc_ifthen_rightbar hr {
						margin-top: 2em;
						margin-bottom: 1em;
					}
					#wc_ifthen_rightbar iframe {
						width: 100%;
						border: none;
						margin: 0;
						padding: 0;
						height: 250px;
						overflow: hidden;
					}
					#wc_ifthen_rightbar h4:first-child {
						margin-top: 0px;
					}
					#wc_ifthen_rightbar p {
					}
					#wc_ifthen_rightbar p img {
						max-width: 100%;
						height: auto;
					}
				}
				.wc_ifthen_list {
					list-style-type: disc;
					list-style-position: inside;
				}
				.wc_ifthen_list li {
					margin-left: 1.5em;
				}
				#wc_ifthen_callback_div {
					display: none;
					background-color: #fff;
					transition: background-color 0.5s ease;
					max-width: 90%;
					margin: auto;
					padding: 20px;
				}
				#wc_ifthen_callback_div.focus {
					background-color: #FFEFBF;
				}
			</style>
			<?php
		}

		public function send_callback_email() {
			if ( isset( $_POST['wc_ifthen_callback_send'] ) && intval( $_POST['wc_ifthen_callback_send'] )==1) {
				$to = WC_IfthenPay_Webdados()->mbway_callback_email;
				$cc = get_option( 'admin_email' );
				$subject = 'Activação de Callback MB WAY (Key: '.$this->mbwaykey.')';
				$message = 'Por favor activar Callback MB WAY com os seguintes dados:

MB WAY Key:
'.$this->mbwaykey.'

Chave anti-phishing (MB WAY):
'.$this->secret_key.'

URL:
'.WC_IfthenPay_Webdados()->mbway_notify_url.'

Email enviado automaticamente do plugin WordPress "Multibanco and MBWAY (IfthenPay gateway) for WooCommerce" para '.$to.' com CC para '.$cc;
				$headers = array(
					'From: '.get_option( 'admin_email' ).' <'.get_option( 'admin_email' ).'>',
					'Cc: '.$cc
				);
				wp_mail( $to, $subject, $message, $headers );
				update_option( $this->id . '_callback_email_sent', 'yes' );
				WC_Admin_Settings::add_message( __( 'The "Callback" activation request has been submited to IfthenPay. Wait for their feedback.', 'multibanco-ifthen-software-gateway-for-woocommerce' ) );
			}
		}

		/**
		 * Icon HTML
		 */
		public function get_icon() {
			$alt = ( WC_IfthenPay_Webdados()->wpml_active ? icl_t( $this->id, $this->id.'_title', $this->title ) : $this->title );
			$icon_html = ( $this->get_option( 'small_icon' ) == 'yes' ? '<img src="'.esc_attr( $this->icon ).'" alt="'.esc_attr( $alt ).'" width="26" height="24"/>' : '<img src="'.esc_attr( $this->icon ).'" alt="'.esc_attr( $alt ).'" width="50" height="24"/>' );
			return apply_filters( 'woocommerce_gateway_icon', $icon_html, $this->id );
		}

		/**
		 * Thank you page
		 */
		function thankyou( $order_id ) {
			if ( is_object( $order_id ) ) {
				$order_id = version_compare( WC_VERSION, '3.0', '>=' ) ? $order_id->get_id() : $order_id->id;
			}
			$order = new WC_Order_MB_Ifthen( $order_id );
			if ( $this->id === $order->mb_get_payment_method() ) {
				if ( $order->mb_has_status( 'on-hold' ) || $order->mb_has_status( 'pending' ) ) {
					if ( date_i18n( 'Y-m-d H:i:s', strtotime( '-'.( WC_IfthenPay_Webdados()->mbway_minutes * WC_IfthenPay_Webdados()->mbway_multiplier_new_payment ).' MINUTES', current_time( 'timestamp' ) ) ) > $order->mb_get_meta( '_'.WC_IfthenPay_Webdados()->mbway_id.'_time' ) ) {
						//Expired
						$expired = true;
						echo $this->thankyou_instructions_table_html_expired( $order_id, round( $order->mb_get_total(), 2 ) ); //Missing MB WAY email or phone number?
					} else {
						//Not expired
						$expired = false;
						echo $this->thankyou_instructions_table_html( $order_id, round( $order->mb_get_total(), 2 ) ); //Missing MB WAY email or phone number?
					}
					//Another payment option
					if ( $expired || apply_filters( 'mbway_ifthen_enable_pay_another_method_thankyou', true, $order_id ) ) {
						?>
						<p class="<?php echo $this->id; ?>_pay_another_method <?php echo $this->id; ?>_text_small">
							<a href="<?php echo esc_url( $order->get_checkout_payment_url() ); ?>" class="button"><?php _e( 'Click here if you wish to use another payment method', 'multibanco-ifthen-software-gateway-for-woocommerce' ) ?></a>
						</p>
						<?php
					}
					if ( is_wc_endpoint_url( 'order-received' ) ) {
						if ( apply_filters( 'mbway_ifthen_enable_check_order_status_thankyou', true, $order_id ) ) { // return false to mbway_ifthen_enable_check_order_status_thankyou in order to stop the ajax checking
							//Check order status
							?>
							<input type="hidden" id="mbway-order-id" value="<?php echo intval( $order_id ); ?>"/>
							<input type="hidden" id="mbway-order-key" value="<?php echo esc_attr( $order->mb_get_order_key() ); ?>"/>
							<?php
							wp_enqueue_script( 'mbway-ifthenpay', plugins_url( 'js/mbway.js', __FILE__ ) , array( 'jquery' ), WP_DEBUG ? rand(0,99999) : $this->version, true );
						}
					}
				} else {
					//Processing
					if ( $order->mb_has_status( 'processing' ) && !is_wc_endpoint_url( 'view-order') ) {
						echo $this->email_instructions_payment_received( $order_id );
					}
				}
			}
		}
		function thankyou_instructions_table_html_css() {
			ob_start();
			?>
			<style type="text/css">
				table.<?php echo $this->id; ?>_table {
					width: auto !important;
					margin: auto;
					margin-top: 2em;
					margin-bottom: 2em;
				}
				table.<?php echo $this->id; ?>_table td,
				table.<?php echo $this->id; ?>_table th {
					background-color: #FFFFFF;
					color: #000000;
					padding: 10px;
					vertical-align: middle;
					white-space: nowrap;
				}
				@media only screen and (max-width: 450px)  {
					table.<?php echo $this->id; ?>_table td,
					table.<?php echo $this->id; ?>_table th {
						white-space: normal;
					}
				}
				table.<?php echo $this->id; ?>_table th {
					text-align: center;
					font-weight: bold;
				}
				table.<?php echo $this->id; ?>_table th img {
					margin: auto;
					margin-top: 10px;
				}
				.<?php echo $this->id; ?>_text_small {
					font-size: small;
				}
				p.<?php echo $this->id; ?>_pay_another_method {
					text-align: center;
					margin-top: 0px;
					margin-bottom: 3em;
				}
			</style>
			<?php
			return ob_get_clean();
		}
		function thankyou_instructions_table_html( $order_id, $order_total ) { //Missing MB WAY email or phone number?
			$alt = ( WC_IfthenPay_Webdados()->wpml_active ? icl_t( $this->id, $this->id.'_title', $this->title ) : $this->title );
			$extra_instructions = ( WC_IfthenPay_Webdados()->wpml_active ? icl_t( $this->id, $this->id.'_extra_instructions', $this->extra_instructions ) : $this->extra_instructions );
			ob_start();
			echo $this->thankyou_instructions_table_html_css();
			?>
			<table class="<?php echo $this->id; ?>_table" cellpadding="0" cellspacing="0">
				<tr>
					<th colspan="2">
						<?php _e( 'Payment instructions', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>
						<br/>
						<img src="<?php echo plugins_url( 'images/banner_mbway.png', __FILE__ ); ?>" alt="<?php echo esc_attr( $alt); ?>" title="<?php echo esc_attr( $alt); ?>"/>
					</th>
				</tr>
				<tr>
					<td><?php _e( 'Information', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>:</td>
					<td><?php echo apply_filters( 'mbway_ifthen_webservice_desc', get_bloginfo( 'name' ).' #'.$order_id, $order_id ); ?></td>
				</tr>
				<tr>
					<td><?php _e( 'Value', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>:</td>
					<td><?php echo round( $order_total, 2 ); ?> &euro;</td>
				</tr>
				<tr>
					<td colspan="2" class="<?php echo $this->id; ?>_text_small"><?php echo nl2br( $extra_instructions ); ?><br/><?php
						printf(
							__( 'You only have %d minutes to approve the payment.', 'multibanco-ifthen-software-gateway-for-woocommerce' ),
							WC_IfthenPay_Webdados()->mbway_minutes
						);
					?></td>
				</tr>
			</table>
			<?php
			return apply_filters( 'mbway_ifthen_thankyou_instructions_table_html', ob_get_clean(), round( $order_total, 2 ), $order_id ); //Missing MB WAY email or phone number?
		}
		function thankyou_instructions_table_html_expired( $order_id, $order_total ) { //Missing MB WAY email or phone number?
			$alt = ( WC_IfthenPay_Webdados()->wpml_active ? icl_t( $this->id, $this->id.'_title', $this->title ) : $this->title );
			$order = new WC_Order_MB_Ifthen( $order_id );
			ob_start();
			echo $this->thankyou_instructions_table_html_css();
			?>
			<table class="<?php echo $this->id; ?>_table" cellpadding="0" cellspacing="0">
				<tr>
					<th colspan="2">
						<?php _e( 'Payment instructions', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>
						<br/>
						<img src="<?php echo plugins_url( 'images/banner_mbway.png', __FILE__ ); ?>" alt="<?php echo esc_attr( $alt); ?>" title="<?php echo esc_attr( $alt); ?>"/>
					</th>
				</tr>
				<tr>
					<td><?php _e( 'Information', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>:</td>
					<td><?php echo apply_filters( 'mbway_ifthen_webservice_desc', get_bloginfo( 'name' ).' #'.$order_id, $order_id ); ?></td>
				</tr>
				<tr>
					<td colspan="2" class="<?php echo $this->id; ?>_text_small"><?php printf(
						__( 'The payment request has expired. %1$sPlease try again%2$s.', 'multibanco-ifthen-software-gateway-for-woocommerce' ),
						'<a href="'.esc_url( $order->get_checkout_payment_url() ).'">',
						'</a>'
					); ?></td>
				</tr>
			</table>
			<?php
			return apply_filters( 'mbway_ifthen_thankyou_instructions_table_html_expired', ob_get_clean(), round( $order_total, 2 ), $order_id ); //Missing MB WAY email or phone number?
		}
		function order_details_after_order_table( $order ) {
			if( is_wc_endpoint_url( 'view-order' ) ) {
				$this->thankyou( $order );
			}
		}
		



		/**
		 * Email instructions
		 */
		function email_instructions_1( $order, $sent_to_admin, $plain_text ) { //"Hyyan WooCommerce Polylang" Integration removes "email_instructions" so we use "email_instructions_1"
			$this->email_instructions( $order, $sent_to_admin, $plain_text );
		}
		function email_instructions( $order, $sent_to_admin, $plain_text ) {
			//Avoid duplicate email instructions on some edge cases
			$send = false;
			if ( ( $sent_to_admin ) ) {
			//if ( ( $sent_to_admin ) && ( !WC_IfthenPay_Webdados()->instructions_sent_to_admin ) ) { //Fixed by checking class instances
				//WC_IfthenPay_Webdados()->instructions_sent_to_admin = true;
				$send = true;
			} else {
				if ( ( !$sent_to_admin ) ) {
				//if ( ( !$sent_to_admin ) && ( !WC_IfthenPay_Webdados()->instructions_sent_to_client ) ) { //Fixed by checking class instances
					//WC_IfthenPay_Webdados()->instructions_sent_to_client = true;
					$send = true;
				}
			}
			//$this->debug_log( 'Email instructions send: '.( $send ? 'true' : 'false' ) );
			//Send
			if ( $send ) {
				$order_id = version_compare( WC_VERSION, '3.0', '>=' ) ? $order->get_id() : $order->id;
				$order = new WC_Order_MB_Ifthen( $order_id );
				//Go
				if ( $this->id === $order->mb_get_payment_method() ) {
					$show = false;
					if ( !$sent_to_admin ) {
						$show = true;
					} else {
						if ( $this->send_to_admin ) {
							$show = true;
						}
					}
					if ( $show ) {
						//WPML - Force correct language (?)
						if ( WC_IfthenPay_Webdados()->wpml_active ) {
							global $sitepress;
							if ( $sitepress ) {
								$lang = $order->mb_get_meta( 'wpml_language' );
								if( !empty( $lang ) ){
									WC_IfthenPay_Webdados()->change_email_language( $lang );
								}
							}
						}
						//On Hold or pending
						if ( $order->mb_has_status( 'on-hold' ) || $order->mb_has_status( 'pending' ) ) {
							if ( WC_IfthenPay_Webdados()->wc_deposits_active && $order->mb_get_status() == 'partially-paid' ) {
								//WooCommerce deposits - No instructions
							} else {
								$ref = WC_IfthenPay_Webdados()->multibanco_get_ref( $order_id );
								if ( apply_filters( 'mbway_ifthen_email_instructions_pending_send', true, $order_id ) ) {
									echo $this->email_instructions_table_html( $order_id, round( $order->mb_get_total(), 2 ) ); //Missing MB WAY email or phone number?
								}
							}
						} else {
							//Processing
							if ( $order->mb_has_status( 'processing' ) ) {
								if ( apply_filters( 'mbway_ifthen_email_instructions_payment_received_send', true, $order_id ) ) {
									echo $this->email_instructions_payment_received( $order_id );
								}
							}
						}
					}
					//$this->debug_log( 'Email instructions show: '.( $show ? 'true' : 'false' ) );
				}
			}
		}
		function email_instructions_table_html( $order_id, $order_total ) {  //Missing MB WAY email or phone number?
			$alt = ( WC_IfthenPay_Webdados()->wpml_active ? icl_t( $this->id, $this->id.'_title', $this->title ) : $this->title );
			$extra_instructions = ( WC_IfthenPay_Webdados()->wpml_active ? icl_t( $this->id, $this->id.'_extra_instructions', $this->extra_instructions ) : $this->extra_instructions );
			ob_start();
			?>
			<table cellpadding="10" cellspacing="0" align="center" border="0" style="margin: auto; margin-top: 2em; margin-bottom: 2em; border-collapse: collapse; border: 1px solid #1465AA; border-radius: 4px !important; background-color: #FFFFFF;">
				<tr>
					<td style="border: 1px solid #1465AA; border-top-right-radius: 4px !important; border-top-left-radius: 4px !important; text-align: center; color: #000000; font-weight: bold;" colspan="2">
						<?php _e( 'Payment instructions', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>
						<br/>
						<img src="<?php echo plugins_url( 'images/banner_mbway.png', __FILE__ ); ?>" alt="<?php echo esc_attr( $alt); ?>" title="<?php echo esc_attr( $alt); ?>" style="margin-top: 10px;"/>
					</td>
				</tr>
				<tr>
					<td style="border: 1px solid #1465AA; color: #000000;"><?php _e( 'Information', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>:</td>
					<td style="border: 1px solid #1465AA; color: #000000; white-space: nowrap;"><?php echo apply_filters( 'mbway_ifthen_webservice_desc', get_bloginfo( 'name' ).' #'.$order_id, $order_id ); ?></td>
				</tr>
				<tr>
					<td style="border: 1px solid #1465AA; color: #000000;"><?php _e( 'Value', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>:</td>
					<td style="border: 1px solid #1465AA; color: #000000; white-space: nowrap;"><?php echo round( $order_total, 2 ); ?> &euro;</td>
				</tr>
				<tr>
					<td style="font-size: x-small; border: 1px solid #1465AA; border-bottom-right-radius: 4px !important; border-bottom-left-radius: 4px !important; color: #000000; text-align: center;" colspan="2"><?php echo nl2br( $extra_instructions ); ?><br/><?php
						printf(
							__( 'You only have %d minutes to approve payment', 'multibanco-ifthen-software-gateway-for-woocommerce' ),
							WC_IfthenPay_Webdados()->mbway_minutes
						);
					?></td>
				</tr>
			</table>
			<?php
			return apply_filters( 'mbway_ifthen_email_instructions_table_html', ob_get_clean(), round( $order_total, 2 ), $order_id ); //Missing MB WAY email or phone number?
		}
		function email_instructions_payment_received( $order_id ) {
			$alt = ( WC_IfthenPay_Webdados()->wpml_active ? icl_t( $this->id, $this->id.'_title', $this->title ) : $this->title );
			ob_start();
			?>
			<p style="text-align: center; margin: auto; margin-top: 2em; margin-bottom: 2em;">
				<img src="<?php echo plugins_url( 'images/banner_mbway.png', __FILE__ ); ?>" alt="<?php echo esc_attr( $alt ); ?>" title="<?php echo esc_attr( $alt ); ?>" style="margin: auto;"/>
				<br/>
				<strong><?php _e( 'MB WAY payment received.', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?></strong>
				<br/>
				<?php _e( 'We will now process your order.', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>
			</p>
			<?php
			return apply_filters( 'mbway_ifthen_email_instructions_payment_received', ob_get_clean(), $order_id );
		}

		/**
		 * SMS instructions - General. Can be used to feed any SMS gateway/plugin
		 */
		/*function sms_instructions( $message, $order_id ) {
			$order = new WC_Order_MB_Ifthen( $order_id );
			$instructions = ''; //We return an empty string so that we always replace our placeholder, even if it's not our gateway
			if ( $order->mb_get_payment_method() == $this->id ) {
				switch ( $order->mb_get_status() ) {
				case 'on-hold':
				case 'pending':
					$ref = WC_IfthenPay_Webdados()->multibanco_get_ref( $order_id );
					if ( is_array( $ref) ) {
						$instructions = 
							__( 'Ent', 'multibanco-ifthen-software-gateway-for-woocommerce' )
							.' '
							.$ref['ent']
							.' '
							.__( 'Ref', 'multibanco-ifthen-software-gateway-for-woocommerce' )
							.' '.WC_IfthenPay_Webdados()->format_multibanco_ref( $ref['ref'] )
							.' '
							.__( 'Val', 'multibanco-ifthen-software-gateway-for-woocommerce' )
							.' '
							.$order->mb_get_total();
						$instructions = preg_replace( '/\s+/', ' ', $instructions ); //Remove extra spaces if necessary
						//Filters in case the website owner wants to customize the message
						$instructions = apply_filters( 'mbway_ifthen_sms_instructions', $instructions, $ref['ent'], $ref['ref'], $order->mb_get_total() );
					} else {
						//error getting ref
					}
					break;
				case 'processing':
					//No instructions
					break;
				default:
					return;
					break;
				}
			}
			return $instructions;
		}*/
		/**
		 * SMS instructions for APG SMS Notifications
		 */
		/*function sms_instructions_apg( $message, $order_id ) {
			$replace = $this->sms_instructions( $message, $order_id ); //Get instructions
			return trim( str_replace( '%multibanco_ifthen%', $replace, $message ) ); //Return message with %multibanco_ifthen% replaced by the instructions
		}*/
		/**
		 * SMS instructions for Twilio SMS Notifications
		 */
		//function sms_instructions_twilio( $message, $order_id ) {
		//	return WC_IfthenPay_Webdados()->multibanco_sms_instructions( $message, $order_id ); //Same as APG
		//}
		/**
		 * SMS instructions for Twilio SMS Notifications
		 */
		//function sms_instructions_yith( $placeholders, $order ) {
		//	if ( is_array($placeholders) ) {
		//		$order_id = version_compare( WC_VERSION, '3.0', '>=' ) ? $order->get_id() : $order->id;
		//		$placeholders['{multibanco_ifthen}'] = WC_IfthenPay_Webdados()->multibanco_sms_instructions( '', $order_id );
		//	}
		//	return $placeholders;
		//}

		/**
		 * Webservice SetPedido
		 */
		function webservice_filter_descricao( $desc ) {
			//Trim and decode
			$desc = htmlspecialchars_decode( trim( $desc ), ENT_QUOTES );
			//Remove '
			$desc = str_replace( "'", "", $desc );
			//Remove "
			$desc = str_replace( '"', '', $desc );
			//Limit
			$desc = trim( $desc );
			$desc = substr( $desc, 0, MBWAY_IFTHEN_DESC_LEN );
			return $desc;
		}
		function webservice_set_pedido( $order_id, $phone ) {
			$url = $this->webservice_url.'/SetPedido';
			$order = new WC_Order_MB_Ifthen( $order_id );
			$mbwaykey = apply_filters( 'multibanco_ifthen_base_mbwaykey', $this->mbwaykey, $order );
			$desc = trim( get_bloginfo( 'name' ) );
			$desc = substr( $desc, 0, MBWAY_IFTHEN_DESC_LEN - strlen( ' #'.$order_id ) );
			$desc .= ' #'.$order_id;
			$args = array(
				'method'	=> 'POST',
				'blocking'	=> true,
				'body'		=> array(
					'MbWayKey'		=> $mbwaykey,
					'canal'			=> '03', //Online
					'referencia'	=> (string)$order_id,
					'valor'			=> (string)round( floatval( $order->mb_get_total() ), 2 ),
					'nrtlm'			=> $phone,
					'email'			=> '', //Não usamos
					'descricao'		=> $this->webservice_filter_descricao( apply_filters( 'mbway_ifthen_webservice_desc', $desc, $order_id ) ),
				),
			);
			$response = wp_remote_post( $url, $args );
			if ( is_wp_error( $response ) ) {
				$debug_msg = '- Error contacting the IfthenPay servers - Order '.$order->mb_get_id().' - '.$response->get_error_message();
				$debug_msg_email = $debug_msg.' - Args: '.serialize( $args ).' - Response: '.serialize( $response );
				$this->debug_log( $debug_msg, 'error', true, $debug_msg_email );
				return false;
			} else {
				if ( isset( $response['response']['code'] ) && intval( $response['response']['code'] ) == 200 && isset( $response['body'] ) && trim( $response['body'] ) != '' ) {
					if ( function_exists( 'simplexml_load_string' ) ) {
						$xmlData = simplexml_load_string( $response['body'] );
						if ( trim( $xmlData->IdPedido ) != '' && floatval( $xmlData->Valor ) > 0 ) {
							$id_pedido = trim( $xmlData->IdPedido );
							$valor = floatval( $xmlData->Valor );
							if ( $valor == round( floatval( $order->mb_get_total() ), 2 ) ) {
								WC_IfthenPay_Webdados()->multibanco_set_order_mbway_details( $order_id, array(
									'mbwaykey'  => $mbwaykey,
									'id_pedido' => $id_pedido,
									'phone'     => $phone,
									'val'       => $valor,
								) );
								$this->debug_log( '- MB Way payment request created on IfthenPay servers - Order '.$order->mb_get_id() );
								do_action( 'mbway_ifthen_created_reference', $id_pedido, $order_id, $phone );
								return true;
							} else {
								$debug_msg = '- Error contacting the IfthenPay servers - Order '.$order->mb_get_id().' - Incorrect "Valor"';
								$debug_msg_email = $debug_msg.' - Args: '.serialize( $args ).' - Response: '.serialize( $response );
								$this->debug_log( $debug_msg, 'error', true, $debug_msg_email );
								return false;
							}
						} else {
							$debug_msg = '- Error contacting the IfthenPay servers - Order '.$order->mb_get_id().' - Missing "IdPedido" or "Valor"';
							$debug_msg_email = $debug_msg.' - Args: '.serialize( $args ).' - Response: '.serialize( $response );
							$this->debug_log( $debug_msg, 'error', true, $debug_msg_email );
							return false;
						}
					} else {
						$debug_msg = '- Error contacting the IfthenPay servers - Order '.$order->mb_get_id().' - "simplexml_load_string" function does not exist';
						$this->debug_log( $debug_msg, 'error', true, $debug_msg );
						return false;
					}
				} else {
					$debug_msg = '- Error contacting the IfthenPay servers - Order '.$order->mb_get_id().' - Incorrect response code: '.$response['response']['code'];
					$debug_msg_email = $debug_msg.' - Args: '.serialize( $args ).' - Response: '.serialize( $response );
					$this->debug_log( $debug_msg, 'error', true, $debug_msg_email );
					return false;
				}
			}
			return false;
		}

		/**
		 * Process it
		 */
		function process_payment( $order_id ) {
			//Webservice
			$order = new WC_Order_MB_Ifthen( $order_id );
			$phone = isset( $_POST[$this->id.'_phone'] ) ? trim( sanitize_text_field( $_POST[$this->id.'_phone'] ) ) : '';
			if ( $this->webservice_set_pedido( $order_id, $phone ) ) {
				if ( ! $this->order_initial_status_pending ) {
					// Mark as on-hold
					$order->update_status( 'on-hold', __( 'Awaiting MB WAY payment.', 'multibanco-ifthen-software-gateway-for-woocommerce' ) );
					// Reduce stock levels
					if ( $this->stock_when == 'order' && version_compare( WC_VERSION, '3.4.0', '<' ) ) $order->mb_reduce_order_stock();
				}
				// Remove cart
				WC()->cart->empty_cart();
				// Empty awaiting payment session
				if ( isset( $_SESSION['order_awaiting_payment'] ) ) unset($_SESSION['order_awaiting_payment'] );
				// Return thankyou redirect
				return array(
					'result' => 'success',
					'redirect' => $this->get_return_url( $order )
				);
			} else {
				wc_add_notice( __( 'Error contacting IfthenPay servers to create MB WAY Payment', 'multibanco-ifthen-software-gateway-for-woocommerce' ) , 'error' );
			}
			return;
		}


		/**
		 * Just for €
		 */
		function disable_if_currency_not_euro( $available_gateways ) {
			return WC_IfthenPay_Webdados()->disable_if_currency_not_euro( $available_gateways, $this->id );
		}

		/**
		 * Just for Portugal
		 */
		function disable_unless_portugal( $available_gateways ) {
			return WC_IfthenPay_Webdados()->disable_unless_portugal( $available_gateways, $this->id );
		}

		/**
		 * Just above/bellow certain amounts
		 */
		function disable_only_above_or_bellow( $available_gateways ) {
			return WC_IfthenPay_Webdados()->disable_only_above_or_bellow( $available_gateways, $this->id );
		}


		/* Payment fields */
		function payment_fields() {
			echo wpautop($this->description);
			?>
			<p class="form-row form-row-wide" id="<?php echo $this->id; ?>_phone_field">
				<br/>
				<label for="<?php echo $this->id; ?>_phone">
					<?php _e( 'Phone number linked to MB WAY', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>
					<abbr class="required" title="<?php _e( 'required', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>">*</abbr>
				</label>
				<input type="tel" autocomplete="off" class="input-text" name="<?php echo $this->id; ?>_phone" id="<?php echo $this->id; ?>_phone" placeholder="9xxxxxxxx" maxlength="9"/>
			</p>
			<?php
		}
		function validate_fields() {
			$phone = isset( $_POST[$this->id.'_phone'] ) ? trim( sanitize_text_field( $_POST[$this->id.'_phone'] ) ) : '';
			if ( empty($phone) ) {
				wc_add_notice( sprintf( __( '%s is required', 'multibanco-ifthen-software-gateway-for-woocommerce' ), '<strong>'.__( 'Phone number linked to MB WAY', 'multibanco-ifthen-software-gateway-for-woocommerce' ).'</strong>' ), 'error' );
				return false;
			} else {
				if ( strlen( $phone ) == 9 && substr( $phone, 0, 1 ) == '9' && ctype_digit( $phone ) ) {
					return true;
				} else {
					wc_add_notice( sprintf( __( '%s must be a valid portuguese mobile phone number', 'multibanco-ifthen-software-gateway-for-woocommerce' ), '<strong>'.__( 'Phone number linked to MB WAY', 'multibanco-ifthen-software-gateway-for-woocommerce' ).'</strong>' ), 'error' );
					return false;
				}
			}
		}

		/* Payment complete - Stolen from PayPal method */
		function payment_complete( $order, $txn_id = '', $note = '' ) {
			$order->add_order_note( $note );
			$order->payment_complete( $txn_id );
		}
		/* Reduce stock on 'wc_maybe_reduce_stock_levels'? */
		function woocommerce_payment_complete_reduce_order_stock( $bool, $order_id ) {
			$order = new WC_Order_MB_Ifthen( $order_id );
			if ( $order->mb_get_payment_method() == $this->id ) {
				return ( WC_IfthenPay_Webdados()->woocommerce_payment_complete_reduce_order_stock( $bool, $order_id, $this->id, $this->stock_when ) );
			} else {
				return $bool;
			}
		}

		/**
		 * Callback
		 */
		function callback() {
			@ob_clean();
			//We must 1st check the situation and then process it and send email to the store owner in case of error.
			if (
				isset( $_GET['chave'] )
				&&
				isset( $_GET['referencia'] )
				&&
				isset( $_GET['idpedido'] )
				&&
				isset( $_GET['valor'] )
				&&
				isset( $_GET['estado'] )
			) {
				//Let's process it
				$this->debug_log( '- Callback ('.$_SERVER['REQUEST_URI'].') with all arguments from '.$_SERVER['REMOTE_ADDR'] );
				$referencia = trim( $_GET['referencia'] );
				$id_pedido = str_replace( ' ', '+', trim( $_GET['idpedido'] ) ); //If there's a plus sign on the URL We'll get it as a space, so we need to get it back
				$val = floatval( $_GET['valor'] );
				$estado = trim( $_GET['estado'] );
				$arguments_ok = true;
				$arguments_error = '';
				if ( trim( $_GET['chave'] ) != trim( $this->secret_key ) ) {
					$arguments_ok = false;
					$arguments_error .= ' - Key';
				}
				if ( !is_numeric( $referencia ) ) {
					$arguments_ok = false;
					$arguments_error .= ' - Referencia (numeric)';
				}
				if ( trim( $id_pedido ) == '' ) {
					$arguments_ok = false;
					$arguments_error .= ' - IdPedido';
				}
				if ( !$val >= 1 ) {
					$arguments_ok = false;
					$arguments_error .= ' - Value';
				}
				/*if ( trim( $estado ) != 'PAGO' ) {
					$arguments_ok = false;
					$arguments_error .= ' - Estado';
				}*/
				if ( $arguments_ok ) { //Isto deve ser separado em vários IFs para melhor se identificar o erro
					if ( trim( $estado ) == 'PAGO' ) {
						$orders_exist = false;
						if ( version_compare( WC_VERSION, '3.0', '<' ) ) {
							if ( version_compare( WC_VERSION, '2.2', '<' ) ) {
								//The old way before WooCommerce 2.2
								$args = array(
									'post_type'	=> 'shop_order',
									'post_status' => 'publish',
									'posts_per_page' => -1,
									'tax_query' => array(
										//We should also look for pending because of the filter added on 2017-12-15, but people should not be using WC < 2.2 by now...
										array(
											'taxonomy' => 'shop_order_status',
											'field' => 'slug',
											'terms' => array( 'on-hold', 'pending' ),
										)
									),
									'meta_query' => array(
										array(
											'key' => '_'.$this->id.'_id_pedido',
											'value' => $id_pedido,
											'compare' => 'LIKE'
										)
									)
								);
							} else {
								//The old way From WooCommerce 2.2
								$args = array(
									'post_type' => 'shop_order',
									'post_status' => array( 'wc-on-hold', 'wc-pending' ),
									'posts_per_page' => -1,
									'meta_query' => array(
										array(
											'key' => '_'.$this->id.'_id_pedido',
											'value' => $id_pedido,
											'compare' => 'LIKE'
										)
									)
								);
							}
							$the_query = new WP_Query( $args );
							if ( $the_query->have_posts() ) {
								$orders_exist = true;
								$orders_count = $the_query->post_count;
								while ( $the_query->have_posts() ) : $the_query->the_post();
									$order = new WC_Order_MB_Ifthen( $the_query->post->ID );
								endwhile;
							}
						} else {
							$orders = wc_get_orders( array(
								'type'				=> array( 'shop_order' ),
								'status'			=> array( 'on-hold', 'pending', 'partially-paid' ),
								'limit'				=> -1,
								'_'.$this->id.'_id_pedido' => $id_pedido,
							) );
							if ( count($orders)>0 ) {
								$orders_exist = true;
								$orders_count = count($orders);
								foreach ( $orders as $order ) {
									$order = new WC_Order_MB_Ifthen( $order->get_id() );
								}
							}
						}
						//Order ID?
						if ( $orders_exist && intval( $order->mb_get_id() ) != intval( $referencia ) ) {
							$orders_exist = false;
						}
						if ( $orders_exist ) {
							if ( $orders_count == 1 ) {
								if ( floatval( $val ) == floatval( $order->mb_get_total() ) ) {

									if ( version_compare( WC_VERSION, '2.6', '<' ) ) {
										//We must first change the order status to "pending" and then to "processing" or no email will be sent to the client
										include_once( ABSPATH.'wp-admin/includes/plugin.php' );
										if ( !is_plugin_active( 'order-status-emails-for-woocommerce/order-status-emails-for-woocommerce.php' ) ) //Only if this plugin is not active
											if ( $order->mb_get_status() != 'pending' ) $order->update_status( 'pending', __( 'Temporary status. Used to force an email on the next order status change.', 'multibanco-ifthen-software-gateway-for-woocommerce' ) );
									}
									
									/*if ( $this->stock_when=='' || $this->order_initial_status_pending ) $order->mb_reduce_order_stock();
									$order->update_status( 'processing', __( 'MB WAY payment received.', 'multibanco-ifthen-software-gateway-for-woocommerce' ) ); //Paid */
									$note=__( 'MB WAY payment received.', 'multibanco-ifthen-software-gateway-for-woocommerce' );
									if ( isset( $_GET['datahorapag'] ) && trim( $_GET['datahorapag'] )!='' ) {
										$note.=' '.trim( $_GET['datahorapag'] );
									}
									//WooCommerce Deposits second payment?
									if ( WC_IfthenPay_Webdados()->wc_deposits_active ) {
										if ( $order->mb_get_meta( '_wc_deposits_order_has_deposit' ) == 'yes' ) { //Has deposit
											if ( $order->mb_get_meta( '_wc_deposits_deposit_paid' ) == 'yes' ) { //First payment - OK!
												if ( $order->mb_get_meta( '_wc_deposits_second_payment_paid' ) != 'yes' ) { //Second payment - not ok
													if ( floatval( $order->mb_get_meta( '_wc_deposits_second_payment' ) ) == floatval( $val ) ) { //This really seems like the second payment
														//Set the current order status temporarly back to partially-paid, but first stop the emails
														add_filter( 'woocommerce_email_enabled_customer_partially_paid', '__return_false' );
														add_filter( 'woocommerce_email_enabled_partial_payment', '__return_false' );
														$order->update_status( 'partially-paid', __( 'Temporary status. Used to force WooCommerce Deposits to correctly set the order to processing.', 'multibanco-ifthen-software-gateway-for-woocommerce' ) );
													}
												}
											}
										}
									}
									$this->payment_complete( $order, '', $note );
									do_action( 'mbway_ifthen_callback_payment_complete', $order->mb_get_id() );
									
									header( 'HTTP/1.1 200 OK' );
									$this->debug_log( '-- MB WAY payment received - Order '.$order->mb_get_id(), 'notice', true, 'Callback ('.$_SERVER['HTTP_HOST'].' '.$_SERVER['REQUEST_URI'].') from '.$_SERVER['REMOTE_ADDR'].' - MB WAY payment received' );
									echo 'OK - MB WAY payment received';
								} else {
									header( 'HTTP/1.1 200 OK' );
									$err = 'Error: The value does not match';
									$this->debug_log( '-- '.$err.' - Order '.$order->mb_get_id(), 'warning', true, 'Callback ('.$_SERVER['HTTP_HOST'].' '.$_SERVER['REQUEST_URI'].') from '.$_SERVER['REMOTE_ADDR'].' - The value does not match' );
									echo $err;
									do_action( 'mbway_ifthen_callback_payment_failed', $order->mb_get_id(), $err );
								}
							} else {
								header( 'HTTP/1.1 200 OK' );
								$err = 'Error: More than 1 order found awaiting payment with these details';
								$this->debug_log( '-- '.$err, 'warning', true, 'Callback ('.$_SERVER['HTTP_HOST'].' '.$_SERVER['REQUEST_URI'].') from '.$_SERVER['REMOTE_ADDR'].' - More than 1 order found awaiting payment with these details' );
								echo $err;
								do_action( 'mbway_ifthen_callback_payment_failed', 0, $err );
							}
						} else {
							header( 'HTTP/1.1 200 OK' );
							$err = 'Error: No orders found awaiting payment with these details';
							$this->debug_log( '-- '.$err, 'warning', true, 'Callback ('.$_SERVER['HTTP_HOST'].' '.$_SERVER['REQUEST_URI'].') from '.$_SERVER['REMOTE_ADDR'].' - No orders found awaiting payment with these details' );
							echo $err;
							do_action( 'mbway_ifthen_callback_payment_failed', 0, $err );
						}
					} else {
						header( 'HTTP/1.1 200 OK' );
						$err = 'Error: Cannot process '.trim( $estado ).' status';
						$this->debug_log( '-- '.$err, 'warning', true, 'Callback ('.$_SERVER['HTTP_HOST'].' '.$_SERVER['REQUEST_URI'].') from '.$_SERVER['REMOTE_ADDR'].' - Cannot process '.trim( $estado ).' status' );
						echo $err;
						do_action( 'mbway_ifthen_callback_payment_failed', 0, $err );
					}

				} else {
					//header("Status: 400");
					$err = 'Argument errors';
					$this->debug_log( '-- '.$err, 'warning', true, 'Callback ('.$_SERVER['HTTP_HOST'].' '.$_SERVER['REQUEST_URI'].') with argument errors from '.$_SERVER['REMOTE_ADDR'].$arguments_error );
					do_action( 'mbway_ifthen_callback_payment_failed', 0, $err );
					wp_die( $err, 'WC_MBWAY_IfThen_Webdados', array( 'response' => 500 ) ); //Sends 500
				}
			} else {
				//header("Status: 400");
				$err = 'Callback ('.$_SERVER['REQUEST_URI'].') with missing arguments from '.$_SERVER['REMOTE_ADDR'];
				$this->debug_log( '- '.$ref, 'warning', true, 'Callback ('.$_SERVER['HTTP_HOST'].' '.$_SERVER['REQUEST_URI'].') with missing arguments from '.$_SERVER['REMOTE_ADDR'] );
				do_action( 'mbway_ifthen_callback_payment_failed', 0, $err );
				wp_die( 'Error: Something is missing...', 'WC_MBWAY_IfThen_Webdados', array( 'response' => 500 ) ); //Sends 500
			}
		}

		/* Debug / Log - MOVED TO WC_IfthenPay_Webdados with gateway id as first argument */
		public function debug_log( $message, $level = 'debug', $to_email = false, $email_message = '' ) {
			if ( $this->debug ) {
				WC_IfthenPay_Webdados()->debug_log( $this->id, $message, $level, ( trim( $this->debug_email ) != '' && $to_email ? $this->debug_email : false ) , $email_message );
			}
		}

		/* Global admin notices - For example if callback email activation is still not sent */
		function admin_notices() {
			if (
				trim( $this->enabled ) == 'yes'
				&&
				strlen( trim( $this->mbwaykey ) ) == 10
				&&
				trim( $this->secret_key ) != ''
			) {
				if ( $callback_email_sent = get_option( $this->id . '_callback_email_sent' ) ) { //No notice for older versions
					if ( $callback_email_sent == 'no' ) {
						if ( !isset( $_GET['callback_warning'] ) ) {
							if ( apply_filters( 'mbway_ifthen_show_callback_notice', true ) ) {
								?>
								<div id="mbway_ifthen_callback_notice" class="error notice" style="padding-right: 38px; position: relative;">
									<p>
										<strong><?php _e( 'MB WAY (IfthenPay)', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?></strong>
										<br/>
										<?php _e( 'You haven\'t yet asked IfthenPay for the "Callback" activation. The orders will NOT be automatically updated upon payment.', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>
										<br/>
										<strong><?php _e( 'This is important', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?>! <a href="admin.php?page=wc-settings&amp;tab=checkout&amp;section=wc_mbway_ifthen_webdados&amp;callback_warning=1"><?php _e( 'Do it here', 'multibanco-ifthen-software-gateway-for-woocommerce' ); ?></a>!</strong>
									</p>
								</div>
								<?php
							}
						}
					}
				}
			}
		}

	}
}